#   Copyright 2017 GoDaddy
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

"""Listener action implementation"""


from cliff import lister
from osc_lib.command import command
from osc_lib import exceptions
from osc_lib import utils
from oslo_utils import uuidutils

from tunnelclient.osc.v2 import constants as const
from tunnelclient.osc.v2 import utils as v2_utils
from tunnelclient.osc.v2 import validate

PROTOCOL_CHOICES = ['TCP', 'HTTP', 'HTTPS', 'TERMINATED_HTTPS', 'UDP', 'SCTP']
CLIENT_AUTH_CHOICES = ['NONE', 'OPTIONAL', 'MANDATORY']


class CreateListener(command.ShowOne):
    """Create a listener"""

    def get_parser(self, prog_name):
        parser = super().get_parser(prog_name)

        parser.add_argument(
            'vpnserver',
            metavar='<vpnserver>',
            help="Vpn server for the listener (name or ID)."
        )
        parser.add_argument(
            '--name',
            metavar='<name>',
            help="Set the listener name."
        )
        parser.add_argument(
            '--description',
            metavar='<description>',
            help="Set the description of this listener."
        )
        parser.add_argument(
            '--protocol',
            metavar='{' + ','.join(PROTOCOL_CHOICES) + '}',
            choices=PROTOCOL_CHOICES,
            type=lambda s: s.upper(),  # case insensitive
            #required=True,
            help="The protocol for the listener."
        )
        parser.add_argument(
            '--connection-limit',
            type=int,
            metavar='<limit>',
            help="Set the maximum number of connections permitted for this "
                 "listener."
        )
        parser.add_argument(
            '--default-pool',
            metavar='<pool>',
            help="Set the name or ID of the pool used by the listener if no "
                 "L7 policies match."
        )
        parser.add_argument(
            '--default-tls-container-ref',
            metavar='<container_ref>',
            help="The URI to the key manager service secrets container "
                 "containing the certificate and key for TERMINATED_TLS "
                 "listeners."
        )
        parser.add_argument(
            '--sni-container-refs',
            metavar='<container_ref>',
            nargs='*',
            help="A list of URIs to the key manager service secrets "
                 "containers containing the certificates and keys for "
                 "TERMINATED_TLS the listener using Server Name Indication."
        )
        parser.add_argument(
            '--insert-headers',
            metavar='<header=value,...>',
            help="A dictionary of optional headers to insert into the request "
                 "before it is sent to the backend member."
        )
        parser.add_argument(
            '--protocol-port',
            metavar='<port>',
            #required=True,
            type=int,
            help="Set the protocol port number for the listener."
        )
        parser.add_argument(
            '--timeout-client-data',
            type=int,
            metavar='<timeout>',
            help="Frontend client inactivity timeout in milliseconds. "
                 "Default: 50000."
        )
        parser.add_argument(
            '--timeout-member-connect',
            type=int,
            metavar='<timeout>',
            help="Backend member connection timeout in milliseconds. "
                 "Default: 5000."
        )
        parser.add_argument(
            '--timeout-member-data',
            type=int,
            metavar='<timeout>',
            help="Backend member inactivity timeout in milliseconds. "
                 "Default: 50000."
        )
        parser.add_argument(
            '--timeout-tcp-inspect',
            type=int,
            metavar='<timeout>',
            help="Time, in milliseconds, to wait for additional TCP packets "
                 "for content inspection. Default: 0."
        )
        admin_group = parser.add_mutually_exclusive_group()
        admin_group.add_argument(
            '--enable',
            action='store_true',
            default=True,
            help="Enable listener (default)."
        )
        admin_group.add_argument(
            '--disable',
            action='store_true',
            default=None,
            help="Disable listener."
        )
        parser.add_argument(
            '--client-ca-tls-container-ref',
            metavar='<container_ref>',
            help="The URI to the key manager service secrets container "
                 "containing the CA certificate for TERMINATED_TLS listeners."
        )
        parser.add_argument(
            '--client-authentication',
            metavar='{' + ','.join(CLIENT_AUTH_CHOICES) + '}',
            choices=CLIENT_AUTH_CHOICES,
            type=lambda s: s.upper(),  # case insensitive
            help="The TLS client authentication verify options for "
                 "TERMINATED_TLS listeners."
        )
        parser.add_argument(
            '--client-crl-container-ref',
            metavar='<client_crl_container_ref>',
            help="The URI to the key manager service secrets container "
                 "containting the CA revocation list file for TERMINATED_TLS "
                 "listeners."
        )
        parser.add_argument(
            '--allowed-cidr',
            dest='allowed_cidrs',
            metavar='<allowed_cidr>',
            nargs='?',
            action='append',
            help="CIDR to allow access to the listener (can be set multiple "
                 "times)."
        )
        parser.add_argument(
            '--wait',
            action='store_true',
            help='Wait for action to complete',
        )
        parser.add_argument(
            '--tls-ciphers',
            metavar='<tls_ciphers>',
            help="Set the TLS ciphers to be used "
                 "by the listener in OpenSSL format."
        )
        parser.add_argument(
            '--tls-version',
            dest='tls_versions',
            metavar='<tls_versions>',
            nargs='?',
            action='append',
            help="Set the TLS protocol version to be used "
                 "by the listener (can be set multiple times)."
        )
        parser.add_argument(
            '--alpn-protocol',
            dest='alpn_protocols',
            metavar='<alpn_protocols>',
            nargs='?',
            action='append',
            help="Set the ALPN protocol to be used "
                 "by the listener (can be set multiple times)."
        )

        return parser

    def take_action(self, parsed_args):
        rows = const.LISTENER_ROWS
        attrs = v2_utils.get_listener_attrs(self.app.client_manager,
                                            parsed_args)

        validate.check_listener_attrs(attrs)

        body = {"listener": attrs}
        data = self.app.client_manager.vpn_server.listener_create(
            json=body)

        if parsed_args.wait:
            v2_utils.wait_for_active(
                status_f=(self.app.client_manager.vpn_server.
                          vpn_server_show),
                res_id=data['listener']['vpnservers'][0]['id']
            )
            data = {
                'listener': (
                    self.app.client_manager.vpn_server.listener_show(
                        data['listener']['id']))
            }

        formatters = {'vpnservers': v2_utils.format_list,
                      'pools': v2_utils.format_list,
                      'l7policies': v2_utils.format_list,
                      'insert_headers': v2_utils.format_hash,
                      'allowed_cidrs': v2_utils.format_list_flat}

        return (rows,
                (utils.get_dict_properties(data['listener'],
                                           rows,
                                           formatters=formatters)))


class DeleteListener(command.Command):
    """Delete a listener"""

    def get_parser(self, prog_name):
        parser = super().get_parser(prog_name)

        parser.add_argument(
            'listener',
            metavar="<listener>",
            help="Listener to delete (name or ID)"
        )
        parser.add_argument(
            '--wait',
            action='store_true',
            help='Wait for action to complete',
        )

        return parser

    def take_action(self, parsed_args):
        attrs = v2_utils.get_listener_attrs(self.app.client_manager,
                                            parsed_args)

        listener_id = attrs.pop('listener_id')

        self.app.client_manager.vpn_server.listener_delete(
            listener_id=listener_id)

        if parsed_args.wait:
            v2_utils.wait_for_delete(
                status_f=self.app.client_manager.vpn_server.listener_show,
                res_id=listener_id
            )


class ListListener(lister.Lister):
    """List listeners"""

    def get_parser(self, prog_name):
        parser = super().get_parser(prog_name)

        parser.add_argument(
            '--name',
            metavar='<name>',
            help="List listeners by listener name."
        )
        parser.add_argument(
            '--vpnserver',
            metavar='<vpnserver>',
            help="Filter by vpn server (name or ID).",
        )

        admin_group = parser.add_mutually_exclusive_group()
        admin_group.add_argument(
            '--enable',
            action='store_true',
            default=None,
            help="List enabled listeners."
        )
        admin_group.add_argument(
            '--disable',
            action='store_true',
            default=None,
            help="List disabled listeners."
        )

        parser.add_argument(
            '--project',
            metavar='<project>',
            help="List listeners by project ID."
        )
        return parser

    def take_action(self, parsed_args):
        columns = const.LISTENER_COLUMNS
        attrs = v2_utils.get_listener_attrs(self.app.client_manager,
                                            parsed_args)
        data = self.app.client_manager.vpn_server.listener_list(**attrs)
        formatters = {'vpnservers': v2_utils.format_list}
        return (columns,
                (utils.get_dict_properties(s, columns, formatters=formatters)
                 for s in data['listeners']))


class ShowListener(command.ShowOne):
    """Show the details of a single listener"""

    def get_parser(self, prog_name):
        parser = super().get_parser(prog_name)

        parser.add_argument(
            'listener',
            metavar='<listener>',
            help='Name or UUID of the listener'
        )

        return parser

    def take_action(self, parsed_args):
        rows = const.LISTENER_ROWS
        data = None
        if uuidutils.is_uuid_like(parsed_args.listener):
            try:
                data = self.app.client_manager.vpn_server.listener_show(
                    listener_id=parsed_args.listener)
            except exceptions.NotFound:
                pass
        if data is None:
            attrs = v2_utils.get_listener_attrs(self.app.client_manager,
                                                parsed_args)

            listener_id = attrs.pop('listener_id')

            data = self.app.client_manager.vpn_server.listener_show(
                listener_id=listener_id,
            )
        formatters = {'vpnservers': v2_utils.format_list,
                      'pools': v2_utils.format_list,
                      'l7policies': v2_utils.format_list,
                      'insert_headers': v2_utils.format_hash,
                      'allowed_cidrs': v2_utils.format_list_flat}

        return rows, utils.get_dict_properties(data, rows,
                                               formatters=formatters)


class SetListener(command.Command):
    """Update a listener"""

    def get_parser(self, prog_name):
        parser = super().get_parser(prog_name)

        parser.add_argument(
            'listener',
            metavar="<listener>",
            help="Listener to modify (name or ID)."
        )
        parser.add_argument(
            '--name',
            metavar='<name>',
            help="Set the listener name."
        )
        parser.add_argument(
            '--description',
            metavar='<description>',
            help="Set the description of this listener."
        )
        parser.add_argument(
            '--connection-limit',
            metavar='<limit>',
            help="The maximum number of connections permitted for this "
                 "listener. Default value is -1 which represents infinite "
                 "connections."
        )
        parser.add_argument(
            '--default-pool',
            metavar='<pool>',
            help="The ID of the pool used by the listener if no L7 policies "
                 "match."
        )
        parser.add_argument(
            '--default-tls-container-ref',
            metavar='<container-ref>',
            help="The URI to the key manager service secrets container "
                 "containing the certificate and key for TERMINATED_TLS"
                 "listeners."
        )
        parser.add_argument(
            '--sni-container-refs',
            metavar='<container-ref>',
            nargs='*',
            help="A list of URIs to the key manager service secrets "
                 "containers containing the certificates and keys for "
                 "TERMINATED_TLS the listener using Server Name Indication."
        )
        parser.add_argument(
            '--insert-headers',
            metavar='<header=value>',
            help="A dictionary of optional headers to insert into the request "
                 "before it is sent to the backend member."
        )
        parser.add_argument(
            '--timeout-client-data',
            type=int,
            metavar='<timeout>',
            help="Frontend client inactivity timeout in milliseconds. "
                 "Default: 50000."
        )
        parser.add_argument(
            '--timeout-member-connect',
            type=int,
            metavar='<timeout>',
            help="Backend member connection timeout in milliseconds. "
                 "Default: 5000."
        )
        parser.add_argument(
            '--timeout-member-data',
            type=int,
            metavar='<timeout>',
            help="Backend member inactivity timeout in milliseconds. "
                 "Default: 50000."
        )
        parser.add_argument(
            '--timeout-tcp-inspect',
            type=int,
            metavar='<timeout>',
            help="Time, in milliseconds, to wait for additional TCP packets "
                 "for content inspection. Default: 0."
        )
        admin_group = parser.add_mutually_exclusive_group()
        admin_group.add_argument(
            '--enable',
            action='store_true',
            default=None,
            help="Enable listener."
        )
        admin_group.add_argument(
            '--disable',
            action='store_true',
            default=None,
            help="Disable listener."
        )
        parser.add_argument(
            '--client-ca-tls-container-ref',
            metavar='<container_ref>',
            help="The URI to the key manager service secrets container "
                 "containing the CA certificate for TERMINATED_TLS listeners."
        )
        parser.add_argument(
            '--client-authentication',
            metavar='{' + ','.join(CLIENT_AUTH_CHOICES) + '}',
            choices=CLIENT_AUTH_CHOICES,
            type=lambda s: s.upper(),  # case insensitive
            help="The TLS client authentication verify options for "
                 "TERMINATED_TLS listeners."
        )
        parser.add_argument(
            '--client-crl-container-ref',
            metavar='<client_crl_container_ref>',
            help="The URI to the key manager service secrets container "
                 "containting the CA revocation list file for TERMINATED_TLS "
                 "listeners."
        )
        parser.add_argument(
            '--allowed-cidr',
            dest='allowed_cidrs',
            metavar='<allowed_cidr>',
            nargs='?',
            action='append',
            help="CIDR to allow access to the listener (can be set multiple "
                 "times)."
        )
        parser.add_argument(
            '--wait',
            action='store_true',
            help='Wait for action to complete',
        )
        parser.add_argument(
            '--tls-ciphers',
            metavar='<tls_ciphers>',
            help="Set the TLS ciphers to be used "
                 "by the listener in OpenSSL format."
        )
        parser.add_argument(
            '--tls-version',
            dest='tls_versions',
            metavar='<tls_versions>',
            nargs='?',
            action='append',
            help="Set the TLS protocol version to be used "
                 "by the listener (can be set multiple times)."
        )
        parser.add_argument(
            '--alpn-protocol',
            dest='alpn_protocols',
            metavar='<alpn_protocols>',
            nargs='?',
            action='append',
            help="Set the ALPN protocol to be used "
                 "by the listener (can be set multiple times)."
        )

        return parser

    def take_action(self, parsed_args):
        attrs = v2_utils.get_listener_attrs(self.app.client_manager,
                                            parsed_args)

        listener_id = attrs.pop('listener_id')

        body = {'listener': attrs}

        self.app.client_manager.vpn_server.listener_set(
            listener_id, json=body)

        if parsed_args.wait:
            v2_utils.wait_for_active(
                status_f=self.app.client_manager.vpn_server.listener_show,
                res_id=listener_id
            )


class UnsetListener(command.Command):
    """Clear listener settings"""

    def get_parser(self, prog_name):
        parser = super().get_parser(prog_name)

        parser.add_argument(
            'listener',
            metavar="<listener>",
            help="Listener to modify (name or ID)."
        )
        parser.add_argument(
            '--name',
            action='store_true',
            help="Clear the listener name."
        )
        parser.add_argument(
            '--description',
            action='store_true',
            help="Clear the description of this listener."
        )
        parser.add_argument(
            '--connection-limit',
            action='store_true',
            help="Reset the connection limit to the API default."
        )
        parser.add_argument(
            '--default-pool',
            dest='default_pool_id',
            action='store_true',
            help="Clear the default pool from the listener."
        )
        parser.add_argument(
            '--default-tls-container-ref',
            action='store_true',
            help="Remove the default TLS container reference from the "
                 "listener."
        )
        parser.add_argument(
            '--sni-container-refs',
            action='store_true',
            help="Remove the TLS SNI container references from the listener."
        )
        parser.add_argument(
            '--insert-headers',
            action='store_true',
            help="Clear the insert headers from the listener."
        )
        parser.add_argument(
            '--timeout-client-data',
            action='store_true',
            help="Reset the client data timeout to the API default."
        )
        parser.add_argument(
            '--timeout-member-connect',
            action='store_true',
            help="Reset the member connect timeout to the API default."
        )
        parser.add_argument(
            '--timeout-member-data',
            action='store_true',
            help="Reset the member data timeout to the API default."
        )
        parser.add_argument(
            '--timeout-tcp-inspect',
            action='store_true',
            help="Reset the TCP inspection timeout to the API default."
        )
        parser.add_argument(
            '--client-ca-tls-container-ref',
            action='store_true',
            help="Clear the client CA TLS container reference from the "
                 "listener."
        )
        parser.add_argument(
            '--client-authentication',
            action='store_true',
            help="Reset the client authentication setting to the API default."
        )
        parser.add_argument(
            '--client-crl-container-ref',
            action='store_true',
            help="Clear the client CRL container reference from the listener."
        )
        parser.add_argument(
            '--allowed-cidrs',
            action='store_true',
            help="Clear all allowed CIDRs from the listener."
        )
        parser.add_argument(
            '--tls-versions',
            action='store_true',
            help='Clear all TLS versions from the listener.',
        )
        parser.add_argument(
            '--tls-ciphers',
            action='store_true',
            help='Clear all TLS ciphers from the listener.',
        )
        parser.add_argument(
            '--wait',
            action='store_true',
            help='Wait for action to complete.',
        )
        parser.add_argument(
            '--alpn-protocols',
            action='store_true',
            help="Clear all ALPN protocols from the listener."
        )
        return parser

    def take_action(self, parsed_args):
        unset_args = v2_utils.get_unsets(parsed_args)
        if not unset_args:
            return

        listener_id = v2_utils.get_resource_id(
            self.app.client_manager.vpn_server.listener_list,
            'listeners', parsed_args.listener)

        body = {'listener': unset_args}

        self.app.client_manager.vpn_server.listener_set(
            listener_id, json=body)

        if parsed_args.wait:
            v2_utils.wait_for_active(
                status_f=self.app.client_manager.vpn_server.listener_show,
                res_id=listener_id
            )


class ShowListenerStats(command.ShowOne):
    """Shows the current statistics for a listener."""

    def get_parser(self, prog_name):
        parser = super().get_parser(prog_name)

        parser.add_argument(
            'listener',
            metavar='<listener>',
            help='Name or UUID of the listener'
        )

        return parser

    def take_action(self, parsed_args):
        rows = const.VPN_SERVER_STATS_ROWS
        attrs = v2_utils.get_listener_attrs(self.app.client_manager,
                                            parsed_args)

        listener_id = attrs.pop('listener_id')

        data = self.app.client_manager.vpn_server.listener_stats_show(
            listener_id=listener_id,
        )

        return (rows, (utils.get_dict_properties(
            data['stats'], rows, formatters={})))



# Copyright (c) 2018 China Telecom Corporation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import copy
from unittest import mock

from osc_lib import exceptions

from tunnelclient.osc.v2 import constants
from tunnelclient.osc.v2 import flavorprofile
from tunnelclient.tests.unit.osc.v2 import constants as attr_consts
from tunnelclient.tests.unit.osc.v2 import fakes


class TestFlavorProfile(fakes.TestTunnelClient):

    def setUp(self):
        super().setUp()

        self._flavorprofile = fakes.createFakeResource('flavorprofile')
        self.flavorprofile_info = copy.deepcopy(
            attr_consts.FLAVORPROFILE_ATTRS)
        self.columns = copy.deepcopy(constants.FLAVORPROFILE_COLUMNS)

        self.api_mock = mock.Mock()
        self.api_mock.flavorprofile_list.return_value = copy.deepcopy(
            {'flavorprofiles': [attr_consts.FLAVORPROFILE_ATTRS]})
        lb_client = self.app.client_manager
        lb_client.vpn_server = self.api_mock


class TestFlavorProfileList(TestFlavorProfile):

    def setUp(self):
        super().setUp()
        self.datalist = (tuple(
            attr_consts.FLAVORPROFILE_ATTRS[k] for k in self.columns),)
        self.cmd = flavorprofile.ListFlavorProfile(self.app, None)

    def test_flavorprofile_list_no_options(self):
        arglist = []
        verifylist = []

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = self.cmd.take_action(parsed_args)

        self.api_mock.flavorprofile_list.assert_called_with()
        self.assertEqual(self.columns, columns)
        self.assertEqual(self.datalist, tuple(data))

    def test_flavorprofile_list_with_options(self):
        arglist = ['--name', 'flavorprofile1']
        verifylist = [('name', 'flavorprofile1')]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = self.cmd.take_action(parsed_args)
        self.api_mock.flavorprofile_list.assert_called_with(
            name='flavorprofile1')

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.datalist, tuple(data))


class TestFlavorProfileDelete(TestFlavorProfile):

    def setUp(self):
        super().setUp()
        self.cmd = flavorprofile.DeleteFlavorProfile(self.app, None)

    def test_flavorprofile_delete(self):
        arglist = [self._flavorprofile.id]
        verifylist = [
            ('flavorprofile', self._flavorprofile.id)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.api_mock.flavorprofile_delete.assert_called_with(
            flavorprofile_id=self._flavorprofile.id)

    def test_flavorprofile_delete_failure(self):
        arglist = ['unknown_flavorprofile']
        verifylist = [
            ('flavorprofile', 'unknown_flavorprofile')
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.assertRaises(exceptions.CommandError, self.cmd.take_action,
                          parsed_args)
        self.assertNotCalled(self.api_mock.flavorprofile_delete)


class TestFlavorProfileCreate(TestFlavorProfile):

    def setUp(self):
        super().setUp()
        self.api_mock.flavorprofile_create.return_value = {
            'flavorprofile': self.flavorprofile_info}
        lb_client = self.app.client_manager
        lb_client.vpn_server = self.api_mock

        self.cmd = flavorprofile.CreateFlavorProfile(self.app, None)

    @mock.patch('tunnelclient.osc.v2.utils.get_flavorprofile_attrs')
    def test_flavorprofile_create(self, mock_client):
        mock_client.return_value = self.flavorprofile_info
        arglist = ['--name', self._flavorprofile.name,
                   '--provider', 'mock_provider',
                   '--flavor-data', '{"mock_key": "mock_value"}']
        verifylist = [
            ('provider', 'mock_provider'),
            ('name', self._flavorprofile.name),
            ('flavor_data', '{"mock_key": "mock_value"}')
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.api_mock.flavorprofile_create.assert_called_with(
            json={'flavorprofile': self.flavorprofile_info})


class TestFlavorProfileShow(TestFlavorProfile):

    def setUp(self):
        super().setUp()
        self.api_mock.flavorprofile_show.return_value = self.flavorprofile_info
        lb_client = self.app.client_manager
        lb_client.vpn_server = self.api_mock

        self.cmd = flavorprofile.ShowFlavorProfile(self.app, None)

    def test_flavorprofile_show(self):
        arglist = [self._flavorprofile.id]
        verifylist = [
            ('flavorprofile', self._flavorprofile.id),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.api_mock.flavorprofile_show.assert_called_with(
            flavorprofile_id=self._flavorprofile.id)


class TestFlavorProfileSet(TestFlavorProfile):

    def setUp(self):
        super().setUp()
        self.cmd = flavorprofile.SetFlavorProfile(self.app, None)

    def test_flavorprofile_set(self):
        arglist = [self._flavorprofile.id, '--name', 'new_name']
        verifylist = [
            ('flavorprofile', self._flavorprofile.id),
            ('name', 'new_name'),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.api_mock.flavorprofile_set.assert_called_with(
            self._flavorprofile.id, json={
                'flavorprofile': {
                    'name': 'new_name'
                }})



#   Copyright 2017 GoDaddy
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

"""Secgroup action implementation"""


from cliff import lister
from osc_lib.command import command
from osc_lib import exceptions
from osc_lib import utils
from oslo_utils import uuidutils

from tunnelclient.osc.v2 import constants as const
from tunnelclient.osc.v2 import utils as v2_utils
from tunnelclient.osc.v2 import validate


class CreateSecgroup(command.ShowOne):
    """Create a secgroup"""

    def get_parser(self, prog_name):
        parser = super().get_parser(prog_name)

        parser.add_argument(
            'vpnserver',
            metavar='<vpnserver>',
            help="Vpn server for the secgroup (name or ID)."
        )
        parser.add_argument(
            '--name',
            metavar='<name>',
            help="Set the secgroup name."
        )
        parser.add_argument(
            '--description',
            metavar='<description>',
            help="Set the description of this secgroup."
        )
        parser.add_argument(
            '--remote-ip',
            metavar='<ip-address>',
            help="The IP address to allow access.",
            required=True
        )
        return parser

    def take_action(self, parsed_args):
        rows = const.SECGROUP_ROWS
        attrs = v2_utils.get_secgroup_attrs(self.app.client_manager,
                                            parsed_args)

        validate.check_secgroup_attrs(attrs)

        body = {"secgroup": attrs}
        data = self.app.client_manager.vpn_server.secgroup_create(
            json=body)

        formatters = {'vpnservers': v2_utils.format_list,
                      'allowed_cidrs': v2_utils.format_list_flat}

        return (rows,
                (utils.get_dict_properties(data['secgroup'],
                                           rows,
                                           formatters=formatters)))


class DeleteSecgroup(command.Command):
    """Delete a secgroup"""

    def get_parser(self, prog_name):
        parser = super().get_parser(prog_name)

        parser.add_argument(
            'secgroup',
            metavar="<secgroup>",
            help="Secgroup to delete (name or ID)"
        )

        return parser

    def take_action(self, parsed_args):
        attrs = v2_utils.get_secgroup_attrs(self.app.client_manager,
                                            parsed_args)

        secgroup_id = attrs.pop('secgroup_id')

        self.app.client_manager.vpn_server.secgroup_delete(
            secgroup_id=secgroup_id)


class ListSecgroup(lister.Lister):
    """List secgroups"""

    def get_parser(self, prog_name):
        parser = super().get_parser(prog_name)

        parser.add_argument(
            '--name',
            metavar='<name>',
            help="List secgroups by secgroup name."
        )
        parser.add_argument(
            '--vpnserver',
            metavar='<vpnserver>',
            help="Filter by vpn server (name or ID).",
        )

        parser.add_argument(
            '--project',
            metavar='<project>',
            help="List secgroups by project ID."
        )
        return parser

    def take_action(self, parsed_args):
        columns = const.SECGROUP_COLUMNS
        attrs = v2_utils.get_secgroup_attrs(self.app.client_manager,
                                            parsed_args)
        data = self.app.client_manager.vpn_server.secgroup_list(**attrs)
        formatters = {'vpnservers': v2_utils.format_list}
        return (columns,
                (utils.get_dict_properties(s, columns, formatters=formatters)
                 for s in data['secgroups']))


class ShowSecgroup(command.ShowOne):
    """Show the details of a single secgroup"""

    def get_parser(self, prog_name):
        parser = super().get_parser(prog_name)

        parser.add_argument(
            'secgroup',
            metavar='<secgroup>',
            help='Name or UUID of the secgroup'
        )

        return parser

    def take_action(self, parsed_args):
        rows = const.SECGROUP_ROWS
        data = None
        if uuidutils.is_uuid_like(parsed_args.secgroup):
            try:
                data = self.app.client_manager.vpn_server.secgroup_show(
                    secgroup_id=parsed_args.secgroup)
            except exceptions.NotFound:
                pass
        if data is None:
            attrs = v2_utils.get_secgroup_attrs(self.app.client_manager,
                                                parsed_args)

            secgroup_id = attrs.pop('secgroup_id')

            data = self.app.client_manager.vpn_server.secgroup_show(
                secgroup_id=secgroup_id,
            )
        formatters = {'vpnservers': v2_utils.format_list,
                      'allowed_cidrs': v2_utils.format_list_flat}

        return rows, utils.get_dict_properties(data, rows,
                                               formatters=formatters)


class SetSecgroup(command.Command):
    """Update a secgroup"""

    def get_parser(self, prog_name):
        parser = super().get_parser(prog_name)

        parser.add_argument(
            'secgroup',
            metavar="<secgroup>",
            help="Secgroup to modify (name or ID)."
        )
        parser.add_argument(
            '--name',
            metavar='<name>',
            help="Set the secgroup name."
        )
        parser.add_argument(
            '--description',
            metavar='<description>',
            help="Set the description of this secgroup."
        )
        return parser

    def take_action(self, parsed_args):
        attrs = v2_utils.get_secgroup_attrs(self.app.client_manager,
                                            parsed_args)

        secgroup_id = attrs.pop('secgroup_id')

        body = {'secgroup': attrs}

        self.app.client_manager.vpn_server.secgroup_set(
            secgroup_id, json=body)


class UnsetSecgroup(command.Command):
    """Clear secgroup settings"""

    def get_parser(self, prog_name):
        parser = super().get_parser(prog_name)

        parser.add_argument(
            'secgroup',
            metavar="<secgroup>",
            help="Secgroup to modify (name or ID)."
        )
        parser.add_argument(
            '--name',
            action='store_true',
            help="Clear the secgroup name."
        )
        parser.add_argument(
            '--description',
            action='store_true',
            help="Clear the description of this secgroup."
        )
        return parser

    def take_action(self, parsed_args):
        unset_args = v2_utils.get_unsets(parsed_args)
        if not unset_args:
            return

        secgroup_id = v2_utils.get_resource_id(
            self.app.client_manager.vpn_server.secgroup_list,
            'secgroups', parsed_args.secgroup)

        body = {'secgroup': unset_args}

        self.app.client_manager.vpn_server.secgroup_set(
            secgroup_id, json=body)


class ShowSecgroupStats(command.ShowOne):
    """Shows the current statistics for a secgroup."""

    def get_parser(self, prog_name):
        parser = super().get_parser(prog_name)

        parser.add_argument(
            'secgroup',
            metavar='<secgroup>',
            help='Name or UUID of the secgroup'
        )

        return parser

    def take_action(self, parsed_args):
        rows = const.VPN_SERVER_STATS_ROWS
        attrs = v2_utils.get_secgroup_attrs(self.app.client_manager,
                                            parsed_args)

        secgroup_id = attrs.pop('secgroup_id')

        data = self.app.client_manager.vpn_server.secgroup_stats_show(
            secgroup_id=secgroup_id,
        )

        return (rows, (utils.get_dict_properties(
            data['stats'], rows, formatters={})))



# Copyright (c) 2018 China Telecom Corporation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""Availabilityzone profile action implementation"""

from cliff import lister
from osc_lib.command import command
from osc_lib import utils

from tunnelclient.osc.v2 import constants as const
from tunnelclient.osc.v2 import utils as v2_utils


class CreateAvailabilityzoneProfile(command.ShowOne):
    """Create an tunnel availability zone profile"""

    def get_parser(self, prog_name):
        parser = super().get_parser(
            prog_name)

        parser.add_argument(
            '--name',
            metavar='<name>',
            required=True,
            help="New tunnel availability zone profile name."
        )
        parser.add_argument(
            '--provider',
            metavar='<provider name>',
            required=True,
            help="Provider name for the availability zone profile."
        )
        parser.add_argument(
            '--availability-zone-data',
            metavar='<availability_zone_data>',
            required=True,
            help="The JSON string containing the availability zone metadata."
        )

        return parser

    def take_action(self, parsed_args):
        rows = const.AVAILABILITYZONEPROFILE_ROWS
        attrs = v2_utils.get_availabilityzoneprofile_attrs(
            self.app.client_manager, parsed_args)
        body = {"availability_zone_profile": attrs}
        client_manager = self.app.client_manager
        data = client_manager.vpn_server.availabilityzoneprofile_create(
            json=body)

        return (rows,
                (utils.get_dict_properties(
                    data['availability_zone_profile'], rows, formatters={})))


class DeleteAvailabilityzoneProfile(command.Command):
    """Delete an availability zone profile"""

    def get_parser(self, prog_name):
        parser = super().get_parser(
            prog_name)

        parser.add_argument(
            'availabilityzoneprofile',
            metavar='<availabilityzone_profile>',
            help="Availability zone profile to delete (name or ID)"
        )

        return parser

    def take_action(self, parsed_args):
        attrs = v2_utils.get_availabilityzoneprofile_attrs(
            self.app.client_manager, parsed_args)
        availabilityzoneprofile_id = attrs.pop('availability_zone_profile_id')

        self.app.client_manager.vpn_server.availabilityzoneprofile_delete(
            availabilityzoneprofile_id=availabilityzoneprofile_id)


class ListAvailabilityzoneProfile(lister.Lister):
    """List availability zone profiles"""

    def get_parser(self, prog_name):
        parser = super().get_parser(prog_name)

        parser.add_argument(
            '--name',
            metavar='<name>',
            help="List availabilityzone profiles by profile name."
        )
        parser.add_argument(
            '--provider',
            metavar='<provider_name>',
            help="List availability zone profiles according to their "
                 "provider.",
        )

        return parser

    def take_action(self, parsed_args):
        columns = const.AVAILABILITYZONEPROFILE_COLUMNS
        attrs = v2_utils.get_availabilityzoneprofile_attrs(
            self.app.client_manager, parsed_args)
        client_manager = self.app.client_manager
        data = client_manager.vpn_server.availabilityzoneprofile_list(
            **attrs)
        return (columns,
                (utils.get_dict_properties(s, columns, formatters={})
                 for s in data['availability_zone_profiles']))


class ShowAvailabilityzoneProfile(command.ShowOne):
    """Show the details of a single availability zone profile"""

    def get_parser(self, prog_name):
        parser = super().get_parser(prog_name)

        parser.add_argument(
            'availabilityzoneprofile',
            metavar='<availabilityzone_profile>',
            help="Name or UUID of the availability zone profile to show."
        )

        return parser

    def take_action(self, parsed_args):
        rows = const.AVAILABILITYZONEPROFILE_ROWS
        attrs = v2_utils.get_availabilityzoneprofile_attrs(
            self.app.client_manager, parsed_args)
        availabilityzoneprofile_id = attrs.pop('availability_zone_profile_id')
        client_manager = self.app.client_manager
        data = client_manager.vpn_server.availabilityzoneprofile_show(
            availabilityzoneprofile_id=availabilityzoneprofile_id
        )

        return (rows, (utils.get_dict_properties(
            data, rows, formatters={})))


class SetAvailabilityzoneProfile(command.Command):
    """Update an availability zone profile"""

    def get_parser(self, prog_name):
        parser = super().get_parser(prog_name)

        parser.add_argument(
            'availabilityzoneprofile',
            metavar='<availabilityzone_profile>',
            help='Name or UUID of the availability zone profile to update.'
        )
        parser.add_argument(
            '--name',
            metavar='<name>',
            help="Set the name of the availability zone profile."
        )
        parser.add_argument(
            '--provider',
            metavar='<provider_name>',
            help="Set the provider of the availability zone profile."
        )
        parser.add_argument(
            '--availabilityzone-data',
            metavar='<availabilityzone_data>',
            help="Set the availability zone data of the profile."
        )

        return parser

    def take_action(self, parsed_args):
        attrs = v2_utils.get_availabilityzoneprofile_attrs(
            self.app.client_manager, parsed_args)
        availabilityzoneprofile_id = attrs.pop('availability_zone_profile_id')
        body = {'availability_zone_profile': attrs}

        self.app.client_manager.vpn_server.availabilityzoneprofile_set(
            availabilityzoneprofile_id, json=body)



#   Copyright 2017 GoDaddy
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

VPN_SERVER_ROWS = (
    'admin_state_up',
    'availability_zone',
    'created_at',
    'description',
    'flavor_id',
    'id',
    #'listeners',
    'resources',
    'users',
    'certificates',
    'secgroups',
    'name',
    'certi_verify',
    'operating_status',
    'project_id',
    'provider',
    'provisioning_status',
    'updated_at',
    'vip_address',
    'vip_network_id',
    'vip_port_id',
    'vip_qos_policy_id',
    'vip_subnet_id',
)

VPN_SERVER_COLUMNS = (
    'id',
    'name',
    'project_id',
    'vip_address',
    'provisioning_status',
    'operating_status',
    'provider')

VPN_SERVER_STATS_ROWS = (
    'active_connections',
    'bytes_in',
    'bytes_out',
    'request_errors',
    'total_connections')

LISTENER_ROWS = (
    'admin_state_up',
    'connection_limit',
    'created_at',
    'default_pool_id',
    'default_tls_container_ref',
    'description',
    'id',
    'insert_headers',
    'l7policies',
    'vpnservers',
    'name',
    'operating_status',
    'project_id',
    'protocol',
    'protocol_port',
    'provisioning_status',
    'sni_container_refs',
    'timeout_client_data',
    'timeout_member_connect',
    'timeout_member_data',
    'timeout_tcp_inspect',
    'updated_at',
    'client_ca_tls_container_ref',
    'client_authentication',
    'client_crl_container_ref',
    'allowed_cidrs',
    'tls_ciphers',
    'tls_versions',
    'alpn_protocols')

USER_ROWS = (
    'admin_state_up',
    'created_at',
    'description',
    'id',
    'vpnservers',
    'name',
    'password',
    'operating_status',
    'project_id',
    'provisioning_status',
    'updated_at')

CERTIFICATE_ROWS = (
    'admin_state_up',
    'created_at',
    'description',
    'id',
    'vpnservers',
    'name',
    'client_req',
    'client_crt',
    'operating_status',
    'project_id',
    'provisioning_status',
    'updated_at')

LISTENER_COLUMNS = (
    'id',
    'default_pool_id',
    'name',
    'project_id',
    'protocol',
    'protocol_port',
    'admin_state_up')

USER_COLUMNS = (
    'id',
    'name',
    'project_id',
    'admin_state_up')

CERTIFICATE_COLUMNS = (
    'id',
    'name',
    'project_id',
    'admin_state_up')

POOL_ROWS = (
    'admin_state_up',
    'created_at',
    'description',
    'healthmonitor_id',
    'id',
    'lb_algorithm',
    'listeners',
    'vpnservers',
    'members',
    'name',
    'operating_status',
    'project_id',
    'protocol',
    'provisioning_status',
    'session_persistence',
    'updated_at',
    'tls_container_ref',
    'ca_tls_container_ref',
    'crl_container_ref',
    'tls_enabled',
    'tls_ciphers',
    'tls_versions')

POOL_COLUMNS = (
    'id',
    'name',
    'project_id',
    'provisioning_status',
    'protocol',
    'lb_algorithm',
    'admin_state_up')

MEMBER_ROWS = (
    'address',
    'admin_state_up',
    'created_at',
    'id',
    'name',
    'operating_status',
    'project_id',
    'protocol_port',
    'provisioning_status',
    'subnet_id',
    'updated_at',
    'weight',
    'monitor_port',
    'monitor_address',
    'backup'
)

RESOURCE_ROWS = (
    'admin_state_up',
    'created_at',
    'description',
    'id',
    'vpnservers',
    'name',
    'operating_status',
    'project_id',
    'subnet_id',
    'ip_address',
    'protocol',
    'protocol_port',
    'provisioning_status',
    'updated_at')

SECGROUP_ROWS = (
    'admin_state_up',
    'created_at',
    'description',
    'id',
    'vpnservers',
    'name',
    'remote_ip',
    'operating_status',
    'project_id',
    'provisioning_status',
    'updated_at')

MEMBER_COLUMNS = (
    'id',
    'name',
    'project_id',
    'provisioning_status',
    'address',
    'protocol_port',
    'operating_status',
    'weight')

RESOURCE_COLUMNS = (
    'id',
    'name',
    'project_id',
    'ip_address',
    'protocol',
    'protocol_port',
    'admin_state_up')

SECGROUP_COLUMNS = (
    'id',
    'name',
    'project_id',
    'remote_ip',
    'admin_state_up')

L7POLICY_ROWS = (
    'listener_id',
    'description',
    'admin_state_up',
    'rules',
    'project_id',
    'created_at',
    'provisioning_status',
    'updated_at',
    'redirect_pool_id',
    'redirect_url',
    'redirect_prefix',
    'action',
    'position',
    'id',
    'operating_status',
    'name',
    'redirect_http_code')

L7POLICY_COLUMNS = (
    'id',
    'name',
    'project_id',
    'provisioning_status',
    'action',
    'position',
    'admin_state_up')

L7RULE_ROWS = (
    'created_at',
    'compare_type',
    'provisioning_status',
    'invert',
    'admin_state_up',
    'updated_at',
    'value',
    'key',
    'project_id',
    'type',
    'id',
    'operating_status')

L7RULE_COLUMNS = (
    'id',
    'project_id',
    'provisioning_status',
    'compare_type',
    'type',
    'key',
    'value',
    'invert',
    'admin_state_up')

MONITOR_ROWS = (
    'project_id',
    'name',
    'admin_state_up',
    'pools',
    'created_at',
    'provisioning_status',
    'updated_at',
    'delay',
    'expected_codes',
    'max_retries',
    'http_method',
    'timeout',
    'max_retries_down',
    'url_path',
    'type',
    'id',
    'operating_status',
    'http_version',
    'domain_name'
)

MONITOR_COLUMNS = (
    'id',
    'name',
    'project_id',
    'type',
    'admin_state_up',
)

QUOTA_ROWS = (
    'vpn_server',
    'listener',
    'pool',
    'health_monitor',
    'member',
    'l7policy',
    'l7rule',
)

QUOTA_COLUMNS = (
    'project_id',
    'vpn_server',
    'listener',
    'pool',
    'health_monitor',
    'member',
    'l7policy',
    'l7rule',
)

AMPHORA_ROWS = (
    'id',
    'vpnserver_id',
    'compute_id',
    'lb_network_ip',
    'vrrp_ip',
    'ha_ip',
    'vrrp_port_id',
    'ha_port_id',
    'cert_expiration',
    'cert_busy',
    'role',
    'status',
    'vrrp_interface',
    'vrrp_id',
    'vrrp_priority',
    'cached_zone',
    'created_at',
    'updated_at',
    'image_id',
    'compute_flavor',
)

AMPHORA_COLUMNS = (
    'id',
    'vpnserver_id',
    'status',
    'role',
    'lb_network_ip',
    'ha_ip',
)

AMPHORA_COLUMNS_LONG = (
    'id',
    'vpnserver_id',
    'status',
    'role',
    'lb_network_ip',
    'ha_ip',
    'compute_id',
    'cached_zone',
    'image_id',
)

PROVIDER_COLUMNS = (
    'name',
    'description',
)

PROVIDER_CAPABILITY_COLUMNS = (
    'type',
    'name',
    'description',
)

FLAVOR_ROWS = (
    'id',
    'name',
    'flavor_profile_id',
    'enabled',
    'description',
)

FLAVOR_COLUMNS = (
    'id',
    'name',
    'flavor_profile_id',
    'enabled',
)

FLAVORPROFILE_ROWS = (
    'id',
    'name',
    'provider_name',
    'flavor_data'
)

FLAVORPROFILE_COLUMNS = (
    'id',
    'name',
    'provider_name',
)

AVAILABILITYZONE_ROWS = (
    'name',
    'availability_zone_profile_id',
    'enabled',
    'description',
)

AVAILABILITYZONE_COLUMNS = (
    'name',
    'availability_zone_profile_id',
    'enabled',
)

AVAILABILITYZONEPROFILE_ROWS = (
    'id',
    'name',
    'provider_name',
    'availability_zone_data'
)

AVAILABILITYZONEPROFILE_COLUMNS = (
    'id',
    'name',
    'provider_name',
)

PROVISIONING_STATUS = 'provisioning_status'
STATUS = 'status'

# TCP/UDP port min/max
MIN_PORT_NUMBER = 1
MAX_PORT_NUMBER = 65535

# Member weight min/max
MIN_WEIGHT = 0
MAX_WEIGHT = 256



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#


class TunnelClientException(Exception):
    """The base exception class for all exceptions this library raises."""

    def __init__(self, code, message=None, request_id=None):
        self.code = code
        self.message = message or self.__class__.message
        super().__init__(self.message)
        self.request_id = request_id

    def __str__(self):
        return "%s (HTTP %s) (Request-ID: %s)" % (self.message,
                                                  self.code,
                                                  self.request_id)



#   Copyright 2017 GoDaddy
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

"""Resource action implementation"""


from cliff import lister
from osc_lib.command import command
from osc_lib import exceptions
from osc_lib import utils
from oslo_utils import uuidutils

from tunnelclient.osc.v2 import constants as const
from tunnelclient.osc.v2 import utils as v2_utils
from tunnelclient.osc.v2 import validate

PROTOCOL_CHOICES = ['TCP', 'UDP']

class CreateResource(command.ShowOne):
    """Create a resource"""

    def get_parser(self, prog_name):
        parser = super().get_parser(prog_name)

        parser.add_argument(
            'vpnserver',
            metavar='<vpnserver>',
            help="Vpn server for the resource (name or ID)."
        )
        parser.add_argument(
            '--name',
            metavar='<name>',
            help="Set the resource name."
        )
        parser.add_argument(
            '--description',
            metavar='<description>',
            help="Set the description of this resource."
        )
        parser.add_argument(
            '--subnet-id',
            metavar='<subnet_id>',
            required=True,
            help="The subnet ID the member service is accessible from."
        )
        parser.add_argument(
            '--ip-address',
            metavar='<ip_address>',
            help="The IP address of the resource to add",
            #required=True
        )
        parser.add_argument(
            '--protocol',
            metavar='{' + ','.join(PROTOCOL_CHOICES) + '}',
            choices=PROTOCOL_CHOICES,
            type=lambda s: s.upper(),  # case insensitive
            #required=True,
            help="The protocol for the listener."
        )
        parser.add_argument(
            '--protocol-port',
            metavar='<port>',
            #required=True,
            type=int,
            help="Set the protocol port number for the listener."
        )
        return parser

    def take_action(self, parsed_args):
        rows = const.RESOURCE_ROWS
        attrs = v2_utils.get_resource_attrs(self.app.client_manager,
                                            parsed_args)

        validate.check_resource_attrs(attrs)

        if 'ip_address' not in attrs:
            subnet_id = attrs['subnet_id']
            subnet_info = self.app.client_manager.neutronclient.show_subnet(subnet_id)
            subnet_cidr = subnet_info['subnet']['cidr']
            attrs['ip_address'] = subnet_cidr

        body = {"resource": attrs}
        data = self.app.client_manager.vpn_server.resource_create(
            json=body)

        formatters = {'vpnservers': v2_utils.format_list,
                      'pools': v2_utils.format_list,
                      'allowed_cidrs': v2_utils.format_list_flat}

        return (rows,
                (utils.get_dict_properties(data['resource'],
                                           rows,
                                           formatters=formatters)))


class DeleteResource(command.Command):
    """Delete a resource"""

    def get_parser(self, prog_name):
        parser = super().get_parser(prog_name)

        parser.add_argument(
            'resource',
            metavar="<resource>",
            help="Resource to delete (name or ID)"
        )

        return parser

    def take_action(self, parsed_args):
        attrs = v2_utils.get_resource_attrs(self.app.client_manager,
                                            parsed_args)

        resource_id = attrs.pop('resource_id')

        self.app.client_manager.vpn_server.resource_delete(
            resource_id=resource_id)


class ListResource(lister.Lister):
    """List resources"""

    def get_parser(self, prog_name):
        parser = super().get_parser(prog_name)

        parser.add_argument(
            '--name',
            metavar='<name>',
            help="List resources by resource name."
        )
        parser.add_argument(
            '--vpnserver',
            metavar='<vpnserver>',
            help="Filter by vpn server (name or ID).",
        )

        parser.add_argument(
            '--project',
            metavar='<project>',
            help="List resources by project ID."
        )
        return parser

    def take_action(self, parsed_args):
        columns = const.RESOURCE_COLUMNS
        attrs = v2_utils.get_resource_attrs(self.app.client_manager,
                                            parsed_args)
        data = self.app.client_manager.vpn_server.resource_list(**attrs)
        formatters = {'vpnservers': v2_utils.format_list}
        return (columns,
                (utils.get_dict_properties(s, columns, formatters=formatters)
                 for s in data['resources']))


class ShowResource(command.ShowOne):
    """Show the details of a single resource"""

    def get_parser(self, prog_name):
        parser = super().get_parser(prog_name)

        parser.add_argument(
            'resource',
            metavar='<resource>',
            help='Name or UUID of the resource'
        )

        return parser

    def take_action(self, parsed_args):
        rows = const.RESOURCE_ROWS
        data = None
        if uuidutils.is_uuid_like(parsed_args.resource):
            try:
                data = self.app.client_manager.vpn_server.resource_show(
                    resource_id=parsed_args.resource)
            except exceptions.NotFound:
                pass
        if data is None:
            attrs = v2_utils.get_resource_attrs(self.app.client_manager,
                                                parsed_args)

            resource_id = attrs.pop('resource_id')

            data = self.app.client_manager.vpn_server.resource_show(
                resource_id=resource_id,
            )
        formatters = {'vpnservers': v2_utils.format_list,
                      'allowed_cidrs': v2_utils.format_list_flat}

        return rows, utils.get_dict_properties(data, rows,
                                               formatters=formatters)


class SetResource(command.Command):
    """Update a resource"""

    def get_parser(self, prog_name):
        parser = super().get_parser(prog_name)

        parser.add_argument(
            'resource',
            metavar="<resource>",
            help="Resource to modify (name or ID)."
        )
        parser.add_argument(
            '--name',
            metavar='<name>',
            help="Set the resource name."
        )
        parser.add_argument(
            '--description',
            metavar='<description>',
            help="Set the description of this resource."
        )
        return parser

    def take_action(self, parsed_args):
        attrs = v2_utils.get_resource_attrs(self.app.client_manager,
                                            parsed_args)

        resource_id = attrs.pop('resource_id')

        body = {'resource': attrs}

        self.app.client_manager.vpn_server.resource_set(
            resource_id, json=body)

        if parsed_args.wait:
            v2_utils.wait_for_active(
                status_f=self.app.client_manager.vpn_server.resource_show,
                res_id=resource_id
            )


class UnsetResource(command.Command):
    """Clear resource settings"""

    def get_parser(self, prog_name):
        parser = super().get_parser(prog_name)

        parser.add_argument(
            'resource',
            metavar="<resource>",
            help="Resource to modify (name or ID)."
        )
        parser.add_argument(
            '--name',
            action='store_true',
            help="Clear the resource name."
        )
        parser.add_argument(
            '--description',
            action='store_true',
            help="Clear the description of this resource."
        )
        return parser

    def take_action(self, parsed_args):
        unset_args = v2_utils.get_unsets(parsed_args)
        if not unset_args:
            return

        resource_id = v2_utils.get_resource_id(
            self.app.client_manager.vpn_server.resource_list,
            'resources', parsed_args.resource)

        body = {'resource': unset_args}

        self.app.client_manager.vpn_server.resource_set(
            resource_id, json=body)

        if parsed_args.wait:
            v2_utils.wait_for_active(
                status_f=self.app.client_manager.vpn_server.resource_show,
                res_id=resource_id
            )


class ShowResourceStats(command.ShowOne):
    """Shows the current statistics for a resource."""

    def get_parser(self, prog_name):
        parser = super().get_parser(prog_name)

        parser.add_argument(
            'resource',
            metavar='<resource>',
            help='Name or UUID of the resource'
        )

        return parser

    def take_action(self, parsed_args):
        rows = const.VPN_SERVER_STATS_ROWS
        attrs = v2_utils.get_resource_attrs(self.app.client_manager,
                                            parsed_args)

        resource_id = attrs.pop('resource_id')

        data = self.app.client_manager.vpn_server.resource_stats_show(
            resource_id=resource_id,
        )

        return (rows, (utils.get_dict_properties(
            data['stats'], rows, formatters={})))



#   Copyright 2017 GoDaddy
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

"""User action implementation"""


from cliff import lister
from osc_lib.command import command
from osc_lib import exceptions
from osc_lib import utils
from oslo_utils import uuidutils

from tunnelclient.osc.v2 import constants as const
from tunnelclient.osc.v2 import utils as v2_utils
from tunnelclient.osc.v2 import validate


class CreateUser(command.ShowOne):
    """Create a user"""

    def get_parser(self, prog_name):
        parser = super().get_parser(prog_name)

        parser.add_argument(
            'vpnserver',
            metavar='<vpnserver>',
            help="Vpn server for the user (name or ID)."
        )
        parser.add_argument(
            '--name',
            metavar='<name>',
            required=True,
            help="Set the user name."
        )
        parser.add_argument(
            '--password',
            metavar='<password>',
            required=True,
            help="Password of the user name."
        )
        parser.add_argument(
            '--description',
            metavar='<description>',
            help="Set the description of this user."
        )
        return parser

    def take_action(self, parsed_args):
        rows = const.USER_ROWS
        attrs = v2_utils.get_user_attrs(self.app.client_manager,
                                            parsed_args)

        validate.check_user_attrs(attrs)

        body = {"user": attrs}
        data = self.app.client_manager.vpn_server.user_create(
            json=body)

        formatters = {'vpnservers': v2_utils.format_list}

        return (rows,
                (utils.get_dict_properties(data['user'],
                                           rows,
                                           formatters=formatters)))


class DeleteUser(command.Command):
    """Delete a user"""

    def get_parser(self, prog_name):
        parser = super().get_parser(prog_name)

        parser.add_argument(
            'user',
            metavar="<user>",
            help="User to delete (name or ID)"
        )
        return parser

    def take_action(self, parsed_args):
        attrs = v2_utils.get_user_attrs(self.app.client_manager,
                                            parsed_args)

        user_id = attrs.pop('user_id')

        self.app.client_manager.vpn_server.user_delete(
            user_id=user_id)


class ListUser(lister.Lister):
    """List users"""

    def get_parser(self, prog_name):
        parser = super().get_parser(prog_name)

        parser.add_argument(
            '--name',
            metavar='<name>',
            help="List users by user name."
        )
        parser.add_argument(
            '--vpnserver',
            metavar='<vpnserver>',
            help="Filter by vpn server (name or ID).",
        )

        parser.add_argument(
            '--project',
            metavar='<project>',
            help="List users by project ID."
        )
        return parser

    def take_action(self, parsed_args):
        columns = const.USER_COLUMNS
        attrs = v2_utils.get_user_attrs(self.app.client_manager,
                                            parsed_args)
        data = self.app.client_manager.vpn_server.user_list(**attrs)
        formatters = {'vpnservers': v2_utils.format_list}
        return (columns,
                (utils.get_dict_properties(s, columns, formatters=formatters)
                 for s in data['users']))


class ShowUser(command.ShowOne):
    """Show the details of a single user"""

    def get_parser(self, prog_name):
        parser = super().get_parser(prog_name)

        parser.add_argument(
            'user',
            metavar='<user>',
            help='Name or UUID of the user'
        )

        return parser

    def take_action(self, parsed_args):
        rows = const.USER_ROWS
        data = None
        if uuidutils.is_uuid_like(parsed_args.user):
            try:
                data = self.app.client_manager.vpn_server.user_show(
                    user_id=parsed_args.user)
            except exceptions.NotFound:
                pass
        if data is None:
            attrs = v2_utils.get_user_attrs(self.app.client_manager,
                                                parsed_args)

            user_id = attrs.pop('user_id')

            data = self.app.client_manager.vpn_server.user_show(
                user_id=user_id,
            )
        formatters = {'vpnservers': v2_utils.format_list}

        return rows, utils.get_dict_properties(data, rows,
                                               formatters=formatters)


class SetUser(command.Command):
    """Update a user"""

    def get_parser(self, prog_name):
        parser = super().get_parser(prog_name)

        parser.add_argument(
            'user',
            metavar="<user>",
            help="User to modify (name or ID)."
        )
        parser.add_argument(
            '--password',
            metavar='<password>',
            required=True,
            help="Password of the user name."
        )
        parser.add_argument(
            '--description',
            metavar='<description>',
            help="Set the description of this user."
        )
        return parser

    def take_action(self, parsed_args):
        attrs = v2_utils.get_user_attrs(self.app.client_manager,
                                            parsed_args)

        user_id = attrs.pop('user_id')

        body = {'user': attrs}

        self.app.client_manager.vpn_server.user_set(
            user_id, json=body)


class UnsetUser(command.Command):
    """Clear user settings"""

    def get_parser(self, prog_name):
        parser = super().get_parser(prog_name)

        parser.add_argument(
            'user',
            metavar="<user>",
            help="User to modify (name or ID)."
        )
        parser.add_argument(
            '--name',
            action='store_true',
            help="Clear the user name."
        )
        parser.add_argument(
            '--description',
            action='store_true',
            help="Clear the description of this user."
        )
        return parser

    def take_action(self, parsed_args):
        unset_args = v2_utils.get_unsets(parsed_args)
        if not unset_args:
            return

        user_id = v2_utils.get_resource_id(
            self.app.client_manager.vpn_server.user_list,
            'users', parsed_args.user)

        body = {'user': unset_args}

        self.app.client_manager.vpn_server.user_set(
            user_id, json=body)


class ShowUserStats(command.ShowOne):
    """Shows the current statistics for a user."""

    def get_parser(self, prog_name):
        parser = super().get_parser(prog_name)

        parser.add_argument(
            'user',
            metavar='<user>',
            help='Name or UUID of the user'
        )

        return parser

    def take_action(self, parsed_args):
        rows = const.VPN_SERVER_STATS_ROWS
        attrs = v2_utils.get_user_attrs(self.app.client_manager,
                                            parsed_args)

        user_id = attrs.pop('user_id')

        data = self.app.client_manager.vpn_server.user_stats_show(
            user_id=user_id,
        )

        return (rows, (utils.get_dict_properties(
            data['stats'], rows, formatters={})))



#   Copyright 2019 Red Hat, Inc. All rights reserved.
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
import copy
from unittest import mock

from tunnelclient.osc.v2 import constants
from tunnelclient.osc.v2 import l7rule
from tunnelclient.tests.unit.osc.v2 import constants as attr_consts
from tunnelclient.tests.unit.osc.v2 import fakes


class TestL7Rule(fakes.TestTunnelClient):

    def setUp(self):
        super().setUp()

        self._l7ru = fakes.createFakeResource('l7rule')
        self.l7rule_info = copy.deepcopy(attr_consts.L7RULE_ATTRS)
        self.columns = copy.deepcopy(constants.L7RULE_COLUMNS)
        self._l7po = fakes.createFakeResource('l7policy')

        self.api_mock = mock.Mock()
        self.api_mock.l7rule_list.return_value = copy.deepcopy(
            {'rules': [attr_consts.L7RULE_ATTRS]})
        self.api_mock.l7policy_list.return_value = copy.deepcopy(
            {'l7policies': [attr_consts.L7POLICY_ATTRS]})

        lb_client = self.app.client_manager
        lb_client.vpn_server = self.api_mock


class TestL7RuleList(TestL7Rule):

    def setUp(self):
        super().setUp()
        self.datalist = (tuple(
            attr_consts.L7RULE_ATTRS[k] for k in self.columns),)
        self.cmd = l7rule.ListL7Rule(self.app, None)

    @mock.patch('tunnelclient.osc.v2.utils.get_l7rule_attrs')
    def test_l7rule_list_no_options(self, mock_attrs):
        mock_attrs.return_value = {'l7policy_id': self._l7po.id}
        arglist = [self._l7po.id]
        verifylist = [('l7policy', self._l7po.id)]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = self.cmd.take_action(parsed_args)

        self.api_mock.l7rule_list.assert_called_with(l7policy_id=self._l7po.id)
        self.assertEqual(self.columns, columns)
        self.assertEqual(self.datalist, tuple(data))


class TestL7RuleDelete(TestL7Rule):

    def setUp(self):
        super().setUp()
        self.cmd = l7rule.DeleteL7Rule(self.app, None)

    @mock.patch('tunnelclient.osc.v2.utils.get_l7rule_attrs')
    def test_l7rule_delete(self, mock_attrs):
        mock_attrs.return_value = {
            'l7policy_id': self._l7po.id,
            'l7rule_id': self._l7ru.id,
        }
        arglist = [self._l7po.id, self._l7ru.id]
        verifylist = [
            ('l7policy', self._l7po.id),
            ('l7rule', self._l7ru.id)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.api_mock.l7rule_delete.assert_called_with(
            l7rule_id=self._l7ru.id,
            l7policy_id=self._l7po.id
        )

    @mock.patch('osc_lib.utils.wait_for_delete')
    @mock.patch('tunnelclient.osc.v2.utils.get_l7rule_attrs')
    def test_l7rule_delete_wait(self, mock_attrs, mock_wait):
        mock_attrs.return_value = {
            'l7policy_id': self._l7po.id,
            'l7rule_id': self._l7ru.id,
        }
        arglist = [self._l7po.id, self._l7ru.id, '--wait']
        verifylist = [
            ('l7policy', self._l7po.id),
            ('l7rule', self._l7ru.id),
            ('wait', True),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.api_mock.l7rule_delete.assert_called_with(
            l7rule_id=self._l7ru.id,
            l7policy_id=self._l7po.id
        )
        mock_wait.assert_called_once_with(
            manager=mock.ANY,
            res_id=self._l7ru.id,
            sleep_time=mock.ANY,
            status_field='provisioning_status')


class TestL7RuleCreate(TestL7Rule):

    def setUp(self):
        super().setUp()
        self.api_mock.l7rule_create.return_value = {
            'rule': self.l7rule_info}
        lb_client = self.app.client_manager
        lb_client.vpn_server = self.api_mock

        self.cmd = l7rule.CreateL7Rule(self.app, None)

    @mock.patch('tunnelclient.osc.v2.utils.get_l7rule_attrs')
    def test_l7rule_create(self, mock_attrs):
        mock_attrs.return_value = {
            'l7policy_id': self._l7po.id,
            'compare-type': 'ENDS_WITH',
            'value': '.example.com',
            'type': 'HOST_NAME'
        }
        arglist = [self._l7po.id,
                   '--compare-type', 'ENDS_WITH',
                   '--value', '.example.com',
                   '--type', 'HOST_NAME'.lower()]

        verifylist = [
            ('l7policy', self._l7po.id),
            ('compare_type', 'ENDS_WITH'),
            ('value', '.example.com'),
            ('type', 'HOST_NAME')
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.api_mock.l7rule_create.assert_called_with(
            l7policy_id=self._l7po.id,
            json={'rule': {
                'compare-type': 'ENDS_WITH',
                'value': '.example.com',
                'type': 'HOST_NAME'}
            })

    @mock.patch('osc_lib.utils.wait_for_status')
    @mock.patch('tunnelclient.osc.v2.utils.get_l7rule_attrs')
    def test_l7rule_create_wait(self, mock_attrs, mock_wait):
        mock_attrs.return_value = {
            'l7policy_id': self._l7po.id,
            'compare-type': 'ENDS_WITH',
            'value': '.example.com',
            'type': 'HOST_NAME'
        }
        self.api_mock.l7policy_show.return_value = {
            'listener_id': 'mock_listener_id'}
        self.api_mock.listener_show.return_value = {
            'vpnservers': [{'id': 'mock_lb_id'}]}
        self.api_mock.l7rule_show.return_value = self.l7rule_info
        arglist = [self._l7po.id,
                   '--compare-type', 'ENDS_WITH',
                   '--value', '.example.com',
                   '--type', 'HOST_NAME'.lower(),
                   '--wait']

        verifylist = [
            ('l7policy', self._l7po.id),
            ('compare_type', 'ENDS_WITH'),
            ('value', '.example.com'),
            ('type', 'HOST_NAME'),
            ('wait', True),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.api_mock.l7rule_create.assert_called_with(
            l7policy_id=self._l7po.id,
            json={'rule': {
                'compare-type': 'ENDS_WITH',
                'value': '.example.com',
                'type': 'HOST_NAME'}
            })
        mock_wait.assert_called_once_with(
            status_f=mock.ANY,
            res_id='mock_lb_id',
            sleep_time=mock.ANY,
            status_field='provisioning_status')


class TestL7RuleShow(TestL7Rule):

    def setUp(self):
        super().setUp()
        self.api_mock.l7rule_show.return_value = self.l7rule_info
        lb_client = self.app.client_manager
        lb_client.vpn_server = self.api_mock

        self.cmd = l7rule.ShowL7Rule(self.app, None)

    def test_l7rule_show(self):
        arglist = [self._l7po.id, self._l7ru.id]
        verifylist = [
            ('l7policy', self._l7po.id),
            ('l7rule', self._l7ru.id)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.api_mock.l7rule_show.assert_called_with(
            l7rule_id=self._l7ru.id,
            l7policy_id=self._l7po.id
        )


class TestL7RuleSet(TestL7Rule):

    def setUp(self):
        super().setUp()
        self.cmd = l7rule.SetL7Rule(self.app, None)

    @mock.patch('tunnelclient.osc.v2.utils.get_l7rule_attrs')
    def test_l7rule_set(self, mock_attrs):
        mock_attrs.return_value = {
            'admin_state_up': False,
            'l7policy_id': self._l7po.id,
            'l7rule_id': self._l7ru.id
        }
        arglist = [
            self._l7po.id,
            self._l7ru.id,
            '--disable'
        ]
        verifylist = [
            ('l7policy', self._l7po.id),
            ('l7rule', self._l7ru.id),
            ('disable', True)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.api_mock.l7rule_set.assert_called_with(
            l7rule_id=self._l7ru.id,
            l7policy_id=self._l7po.id,
            json={'rule': {'admin_state_up': False}})

    @mock.patch('osc_lib.utils.wait_for_status')
    @mock.patch('tunnelclient.osc.v2.utils.get_l7rule_attrs')
    def test_l7rule_set_wait(self, mock_attrs, mock_wait):
        mock_attrs.return_value = {
            'admin_state_up': False,
            'l7policy_id': self._l7po.id,
            'l7rule_id': self._l7ru.id
        }
        arglist = [
            self._l7po.id,
            self._l7ru.id,
            '--disable',
            '--wait',
        ]
        verifylist = [
            ('l7policy', self._l7po.id),
            ('l7rule', self._l7ru.id),
            ('disable', True),
            ('wait', True),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.api_mock.l7rule_set.assert_called_with(
            l7rule_id=self._l7ru.id,
            l7policy_id=self._l7po.id,
            json={'rule': {'admin_state_up': False}})
        mock_wait.assert_called_once_with(
            status_f=mock.ANY,
            res_id=self._l7ru.id,
            sleep_time=mock.ANY,
            status_field='provisioning_status')


class TestL7RuleUnset(TestL7Rule):
    PARAMETERS = ('invert', 'key')

    def setUp(self):
        super().setUp()
        self.cmd = l7rule.UnsetL7Rule(self.app, None)

    def test_l7rule_unset_invert(self):
        self._test_l7rule_unset_param('invert')

    def test_l7rule_unset_invert_wait(self):
        self._test_l7rule_unset_param_wait('invert')

    def test_l7rule_unset_key(self):
        self._test_l7rule_unset_param('key')

    def _test_l7rule_unset_param(self, param):
        self.api_mock.l7rule_set.reset_mock()
        arg_param = param.replace('_', '-') if '_' in param else param
        arglist = [self._l7po.id, self._l7ru.id, '--%s' % arg_param]
        ref_body = {'rule': {param: None}}
        verifylist = [
            ('l7rule_id', self._l7ru.id),
        ]
        for ref_param in self.PARAMETERS:
            verifylist.append((ref_param, param == ref_param))
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.api_mock.l7rule_set.assert_called_once_with(
            l7policy_id=self._l7po.id, l7rule_id=self._l7ru.id, json=ref_body)

    @mock.patch('osc_lib.utils.wait_for_status')
    def _test_l7rule_unset_param_wait(self, param, mock_wait):
        self.api_mock.l7rule_set.reset_mock()
        arg_param = param.replace('_', '-') if '_' in param else param
        arglist = [self._l7po.id, self._l7ru.id, '--%s' % arg_param, '--wait']
        ref_body = {'rule': {param: None}}
        verifylist = [
            ('l7policy', self._l7po.id),
            ('l7rule_id', self._l7ru.id),
            ('wait', True),
        ]
        for ref_param in self.PARAMETERS:
            verifylist.append((ref_param, param == ref_param))
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.api_mock.l7rule_set.assert_called_once_with(
            l7policy_id=self._l7po.id, l7rule_id=self._l7ru.id, json=ref_body)
        mock_wait.assert_called_once_with(
            status_f=mock.ANY,
            res_id=self._l7ru.id,
            sleep_time=mock.ANY,
            status_field='provisioning_status')

    def test_l7rule_unset_all(self):
        self.api_mock.l7rule_set.reset_mock()
        ref_body = {'rule': {x: None for x in self.PARAMETERS}}
        arglist = [self._l7po.id, self._l7ru.id]
        for ref_param in self.PARAMETERS:
            arg_param = (ref_param.replace('_', '-') if '_' in ref_param else
                         ref_param)
            arglist.append('--%s' % arg_param)
        verifylist = list(zip(self.PARAMETERS, [True] * len(self.PARAMETERS)))
        verifylist = [('l7rule_id', self._l7ru.id)] + verifylist
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.api_mock.l7rule_set.assert_called_once_with(
            l7policy_id=self._l7po.id, l7rule_id=self._l7ru.id, json=ref_body)

    def test_l7rule_unset_none(self):
        self.api_mock.l7rule_set.reset_mock()
        arglist = [self._l7po.id, self._l7ru.id]
        verifylist = list(zip(self.PARAMETERS, [False] * len(self.PARAMETERS)))
        verifylist = [('l7rule_id', self._l7ru.id)] + verifylist
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.api_mock.l7rule_set.assert_not_called()



#   Copyright 2019 Red Hat, Inc. All rights reserved.
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
import copy
from unittest import mock

from osc_lib import exceptions

from tunnelclient.osc.v2 import constants
from tunnelclient.osc.v2 import l7policy
from tunnelclient.tests.unit.osc.v2 import constants as attr_consts
from tunnelclient.tests.unit.osc.v2 import fakes


class TestL7Policy(fakes.TestTunnelClient):

    def setUp(self):
        super().setUp()

        self._l7po = fakes.createFakeResource('l7policy')
        self.l7po_info = copy.deepcopy(attr_consts.L7POLICY_ATTRS)
        self.columns = copy.deepcopy(constants.L7POLICY_COLUMNS)

        self.api_mock = mock.Mock()
        self.api_mock.l7policy_list.return_value = copy.deepcopy(
            {'l7policies': [attr_consts.L7POLICY_ATTRS]})

        lb_client = self.app.client_manager
        lb_client.vpn_server = self.api_mock


class TestL7PolicyList(TestL7Policy):

    def setUp(self):
        super().setUp()
        self.datalist = (tuple(
            attr_consts.L7POLICY_ATTRS[k] for k in self.columns),)
        self.cmd = l7policy.ListL7Policy(self.app, None)

    def test_l7policy_list_no_options(self):
        arglist = []
        verifylist = []

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = self.cmd.take_action(parsed_args)

        self.api_mock.l7policy_list.assert_called_with()
        self.assertEqual(self.columns, columns)
        self.assertEqual(self.datalist, tuple(data))

    @mock.patch('tunnelclient.osc.v2.utils.get_l7policy_attrs')
    def test_l7policy_list_by_listener(self, mock_l7policy_attrs):
        mock_l7policy_attrs.return_value = {
            'listener_id': self._l7po.listener_id,
        }
        arglist = ['--listener', 'mock_li_id']
        verifylist = [('listener', 'mock_li_id')]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = self.cmd.take_action(parsed_args)

        self.api_mock.l7policy_list.assert_called_with(
            listener_id=self._l7po.listener_id
        )
        self.assertEqual(self.columns, columns)
        self.assertEqual(self.datalist, tuple(data))


class TestL7PolicyDelete(TestL7Policy):

    def setUp(self):
        super().setUp()
        self.cmd = l7policy.DeleteL7Policy(self.app, None)

    def test_l7policy_delete(self):
        arglist = [self._l7po.id]
        verifylist = [
            ('l7policy', self._l7po.id)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.api_mock.l7policy_delete.assert_called_with(
            l7policy_id=self._l7po.id)

    @mock.patch('osc_lib.utils.wait_for_delete')
    def test_l7policy_delete_wait(self, mock_wait):
        arglist = [self._l7po.id, '--wait']
        verifylist = [
            ('l7policy', self._l7po.id),
            ('wait', True),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.api_mock.l7policy_delete.assert_called_with(
            l7policy_id=self._l7po.id)
        mock_wait.assert_called_once_with(
            manager=mock.ANY,
            res_id=self._l7po.id,
            sleep_time=mock.ANY,
            status_field='provisioning_status')

    def test_l7policy_delete_failure(self):
        arglist = ['unknown_policy']
        verifylist = [
            ('l7policy', 'unknown_policy')
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.assertRaises(exceptions.CommandError, self.cmd.take_action,
                          parsed_args)
        self.assertNotCalled(self.api_mock.l7policy_delete)


class TestL7PolicyCreate(TestL7Policy):

    def setUp(self):
        super().setUp()
        self.api_mock.l7policy_create.return_value = {
            'l7policy': self.l7po_info}
        lb_client = self.app.client_manager
        lb_client.vpn_server = self.api_mock

        self.cmd = l7policy.CreateL7Policy(self.app, None)

    @mock.patch('tunnelclient.osc.v2.utils.get_l7policy_attrs')
    def test_l7policy_create(self, mock_attrs):
        mock_attrs.return_value = {
            'listener_id': self._l7po.listener_id,
            'name': self._l7po.name,
            'action': 'REDIRECT_TO_POOL',
            'redirect_pool_id': self._l7po.redirect_pool_id
        }
        arglist = ['mock_li_id',
                   '--name', self._l7po.name,
                   '--action', 'REDIRECT_TO_POOL'.lower(),
                   '--redirect-pool', self._l7po.redirect_pool_id]

        verifylist = [
            ('listener', 'mock_li_id'),
            ('name', self._l7po.name),
            ('action', 'REDIRECT_TO_POOL'),
            ('redirect_pool', self._l7po.redirect_pool_id)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.api_mock.l7policy_create.assert_called_with(
            json={'l7policy': {
                'listener_id': self._l7po.listener_id,
                'name': self._l7po.name,
                'action': 'REDIRECT_TO_POOL',
                'redirect_pool_id': self._l7po.redirect_pool_id
            }})

    @mock.patch('osc_lib.utils.wait_for_status')
    @mock.patch('tunnelclient.osc.v2.utils.get_l7policy_attrs')
    def test_l7policy_create_wait(self, mock_attrs, mock_wait):
        mock_attrs.return_value = {
            'listener_id': self._l7po.listener_id,
            'name': self._l7po.name,
            'action': 'REDIRECT_TO_POOL',
            'redirect_pool_id': self._l7po.redirect_pool_id
        }
        self.api_mock.listener_show.return_value = {
            'vpnservers': [{'id': 'mock_lb_id'}]}
        self.api_mock.l7policy_show.return_value = self.l7po_info
        arglist = ['mock_li_id',
                   '--name', self._l7po.name,
                   '--action', 'REDIRECT_TO_POOL'.lower(),
                   '--redirect-pool', self._l7po.redirect_pool_id,
                   '--wait']

        verifylist = [
            ('listener', 'mock_li_id'),
            ('name', self._l7po.name),
            ('action', 'REDIRECT_TO_POOL'),
            ('redirect_pool', self._l7po.redirect_pool_id),
            ('wait', True),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.api_mock.l7policy_create.assert_called_with(
            json={'l7policy': {
                'listener_id': self._l7po.listener_id,
                'name': self._l7po.name,
                'action': 'REDIRECT_TO_POOL',
                'redirect_pool_id': self._l7po.redirect_pool_id
            }})
        mock_wait.assert_called_once_with(
            status_f=mock.ANY,
            res_id='mock_lb_id',
            sleep_time=mock.ANY,
            status_field='provisioning_status')


class TestL7PolicyShow(TestL7Policy):

    def setUp(self):
        super().setUp()
        self.api_mock.l7policy_list.return_value = [{'id': self._l7po.id}]
        self.api_mock.l7policy_show.return_value = self.l7po_info
        lb_client = self.app.client_manager
        lb_client.vpn_server = self.api_mock

        self.cmd = l7policy.ShowL7Policy(self.app, None)

    @mock.patch('tunnelclient.osc.v2.utils.get_l7policy_attrs')
    def test_l7policy_show(self, mock_attrs):
        mock_attrs.return_value = {'l7policy_id': self._l7po.id}
        arglist = [self._l7po.id]
        verifylist = [
            ('l7policy', self._l7po.id),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.api_mock.l7policy_show.assert_called_with(
            l7policy_id=self._l7po.id)


class TestL7PolicySet(TestL7Policy):

    def setUp(self):
        super().setUp()
        self.cmd = l7policy.SetL7Policy(self.app, None)

    def test_l7policy_set(self):
        arglist = [self._l7po.id, '--name', 'new_name']
        verifylist = [
            ('l7policy', self._l7po.id),
            ('name', 'new_name')
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.api_mock.l7policy_set.assert_called_with(
            self._l7po.id, json={'l7policy': {'name': 'new_name'}})

    @mock.patch('osc_lib.utils.wait_for_status')
    def test_l7policy_set_wait(self, mock_wait):
        arglist = [self._l7po.id, '--name', 'new_name', '--wait']
        verifylist = [
            ('l7policy', self._l7po.id),
            ('name', 'new_name'),
            ('wait', True),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.api_mock.l7policy_set.assert_called_with(
            self._l7po.id, json={'l7policy': {'name': 'new_name'}})
        mock_wait.assert_called_once_with(
            status_f=mock.ANY,
            res_id=self._l7po.id,
            sleep_time=mock.ANY,
            status_field='provisioning_status')


class TestL7PolicyUnset(TestL7Policy):
    PARAMETERS = ('name', 'description', 'redirect_http_code')

    def setUp(self):
        super().setUp()
        self.cmd = l7policy.UnsetL7Policy(self.app, None)

    def test_l7policy_unset_description(self):
        self._test_l7policy_unset_param('description')

    def test_l7policy_unset_name(self):
        self._test_l7policy_unset_param('name')

    def test_l7policy_unset_name_wait(self):
        self._test_l7policy_unset_param_wait('name')

    def test_l7policy_unset_redirect_http_code(self):
        self._test_l7policy_unset_param('redirect_http_code')

    def _test_l7policy_unset_param(self, param):
        self.api_mock.l7policy_set.reset_mock()
        arg_param = param.replace('_', '-') if '_' in param else param
        arglist = [self._l7po.id, '--%s' % arg_param]
        ref_body = {'l7policy': {param: None}}
        verifylist = [
            ('l7policy', self._l7po.id),
        ]
        for ref_param in self.PARAMETERS:
            verifylist.append((ref_param, param == ref_param))
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.api_mock.l7policy_set.assert_called_once_with(
            self._l7po.id, json=ref_body)

    @mock.patch('osc_lib.utils.wait_for_status')
    def _test_l7policy_unset_param_wait(self, param, mock_wait):
        self.api_mock.l7policy_set.reset_mock()
        arg_param = param.replace('_', '-') if '_' in param else param
        arglist = [self._l7po.id, '--%s' % arg_param, '--wait']
        ref_body = {'l7policy': {param: None}}
        verifylist = [
            ('l7policy', self._l7po.id),
            ('wait', True),
        ]
        for ref_param in self.PARAMETERS:
            verifylist.append((ref_param, param == ref_param))
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.api_mock.l7policy_set.assert_called_once_with(
            self._l7po.id, json=ref_body)
        mock_wait.assert_called_once_with(
            status_f=mock.ANY,
            res_id=self._l7po.id,
            sleep_time=mock.ANY,
            status_field='provisioning_status')

    def test_l7policy_unset_all(self):
        self.api_mock.l7policy_set.reset_mock()
        ref_body = {'l7policy': {x: None for x in self.PARAMETERS}}
        arglist = [self._l7po.id]
        for ref_param in self.PARAMETERS:
            arg_param = (ref_param.replace('_', '-') if '_' in ref_param else
                         ref_param)
            arglist.append('--%s' % arg_param)
        verifylist = list(zip(self.PARAMETERS, [True] * len(self.PARAMETERS)))
        verifylist = [('l7policy', self._l7po.id)] + verifylist
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.api_mock.l7policy_set.assert_called_once_with(
            self._l7po.id, json=ref_body)

    def test_l7policy_unset_none(self):
        self.api_mock.l7policy_set.reset_mock()
        arglist = [self._l7po.id]
        verifylist = list(zip(self.PARAMETERS, [False] * len(self.PARAMETERS)))
        verifylist = [('l7policy', self._l7po.id)] + verifylist
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.api_mock.l7policy_set.assert_not_called()



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
import copy
from unittest import mock

import osc_lib.tests.utils as osc_test_utils

from tunnelclient.osc.v2 import constants
from tunnelclient.osc.v2 import member
from tunnelclient.tests.unit.osc.v2 import constants as attr_consts
from tunnelclient.tests.unit.osc.v2 import fakes


class TestMember(fakes.TestTunnelClient):

    def setUp(self):
        super().setUp()

        self._mem = fakes.createFakeResource('member')
        self.mem_info = copy.deepcopy(attr_consts.MEMBER_ATTRS)
        self.columns = copy.deepcopy(constants.MEMBER_COLUMNS)

        info_list = {'members': [
            {k: v for k, v in attr_consts.MEMBER_ATTRS.items() if (
                k in self.columns)}
        ]}
        self.api_mock = mock.Mock()
        self.api_mock.member_list.return_value = info_list
        pool_dict = copy.deepcopy({'pools': [attr_consts.POOL_ATTRS]})
        pool_dict['pools'][0]['id'] = self._mem.pool_id
        self.api_mock.pool_list.return_value = pool_dict

        lb_client = self.app.client_manager
        lb_client.vpn_server = self.api_mock
        lb_client.neutronclient = mock.MagicMock()


class TestListMember(TestMember):

    def setUp(self):
        super().setUp()
        self.datalist = (tuple(
            attr_consts.MEMBER_ATTRS[k] for k in self.columns),)
        self.cmd = member.ListMember(self.app, None)

    def test_member_list_no_options(self):
        arglist = []
        verifylist = []

        self.assertRaises(osc_test_utils.ParserException,
                          self.check_parser, self.cmd, arglist, verifylist)

    @mock.patch('tunnelclient.osc.v2.utils.get_member_attrs')
    def test_member_list(self, mock_attrs):
        mock_attrs.return_value = {'pool_id': 'pool_id',
                                   'project_id': self._mem.project_id}
        arglist = ['pool_id']
        verifylist = []

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        self.api_mock.member_list.assert_called_once_with(pool_id='pool_id')
        self.assertEqual(self.columns, columns)
        self.assertEqual(self.datalist, tuple(data))


class TestCreateMember(TestMember):

    def setUp(self):
        super().setUp()
        self.cmd = member.CreateMember(self.app, None)
        self.api_mock.member_create.return_value = {
            'member': self.mem_info}

    @mock.patch('tunnelclient.osc.v2.utils.get_member_attrs')
    def test_member_create(self, mock_attrs):
        mock_attrs.return_value = {
            'ip_address': '192.0.2.122',
            'protocol_port': self._mem.protocol_port,
            'weight': self._mem.weight,
            'admin_state_up': True,
            'pool_id': self._mem.pool_id,
            'backup': False}

        arglist = ['pool_id', '--address', '192.0.2.122',
                   '--protocol-port', '80',
                   '--weight', '1', '--enable', '--disable-backup']
        verifylist = [
            ('address', '192.0.2.122'),
            ('protocol_port', 80),
            ('weight', 1)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.api_mock.member_create.assert_called_with(
            pool_id=self._mem.pool_id, json={
                'member': {'ip_address': '192.0.2.122',
                           'protocol_port': self._mem.protocol_port,
                           'weight': self._mem.weight,
                           'admin_state_up': True,
                           'backup': False}})

    @mock.patch('osc_lib.utils.wait_for_status')
    @mock.patch('tunnelclient.osc.v2.utils.get_member_attrs')
    def test_member_create_wait(self, mock_attrs, mock_wait):
        mock_attrs.return_value = {
            'ip_address': '192.0.2.122',
            'protocol_port': self._mem.protocol_port,
            'weight': self._mem.weight,
            'admin_state_up': True,
            'pool_id': self._mem.pool_id,
            'backup': False}
        self.api_mock.pool_show.return_value = {
            'vpnservers': [{'id': 'mock_lb_id'}]}
        self.api_mock.member_show.return_value = self.mem_info
        arglist = ['pool_id', '--address', '192.0.2.122',
                   '--protocol-port', '80',
                   '--weight', '1', '--enable', '--disable-backup', '--wait']
        verifylist = [
            ('address', '192.0.2.122'),
            ('protocol_port', 80),
            ('weight', 1),
            ('wait', True),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.api_mock.member_create.assert_called_with(
            pool_id=self._mem.pool_id, json={
                'member': {'ip_address': '192.0.2.122',
                           'protocol_port': self._mem.protocol_port,
                           'weight': self._mem.weight,
                           'admin_state_up': True,
                           'backup': False}})
        mock_wait.assert_called_once_with(
            status_f=mock.ANY,
            res_id='mock_lb_id',
            sleep_time=mock.ANY,
            status_field='provisioning_status')


class TestMemberDelete(TestMember):

    def setUp(self):
        super().setUp()
        self.cmd = member.DeleteMember(self.app, None)

    @mock.patch('tunnelclient.osc.v2.utils.get_member_attrs')
    def test_member_delete(self, mock_attrs):
        mock_attrs.return_value = {'pool_id': self._mem.pool_id,
                                   'member_id': self._mem.id}
        arglist = [self._mem.pool_id, self._mem.id]
        verifylist = [
            ('pool', self._mem.pool_id),
            ('member', self._mem.id),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.api_mock.member_delete.assert_called_with(
            pool_id=self._mem.pool_id, member_id=self._mem.id)

    @mock.patch('osc_lib.utils.wait_for_delete')
    @mock.patch('tunnelclient.osc.v2.utils.get_member_attrs')
    def test_member_delete_wait(self, mock_attrs, mock_wait):
        mock_attrs.return_value = {'pool_id': self._mem.pool_id,
                                   'member_id': self._mem.id}
        arglist = [self._mem.pool_id, self._mem.id, '--wait']
        verifylist = [
            ('pool', self._mem.pool_id),
            ('member', self._mem.id),
            ('wait', True),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.api_mock.member_delete.assert_called_with(
            pool_id=self._mem.pool_id, member_id=self._mem.id)
        mock_wait.assert_called_once_with(
            manager=mock.ANY,
            res_id=self._mem.id,
            sleep_time=mock.ANY,
            status_field='provisioning_status')


class TestMemberSet(TestMember):

    def setUp(self):
        super().setUp()
        self.cmd = member.SetMember(self.app, None)

    @mock.patch('tunnelclient.osc.v2.utils.get_member_attrs')
    def test_member_set(self, mock_attrs):
        mock_attrs.return_value = {'pool_id': self._mem.pool_id,
                                   'member_id': self._mem.id,
                                   'name': 'new_name',
                                   'backup': True}
        arglist = [self._mem.pool_id, self._mem.id, '--name',
                   'new_name', '--enable-backup']
        verifylist = [
            ('pool', self._mem.pool_id),
            ('member', self._mem.id),
            ('name', 'new_name'),
            ('enable_backup', True),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.api_mock.member_set.assert_called_with(
            pool_id=self._mem.pool_id, member_id=self._mem.id,
            json={'member': {'name': 'new_name',
                             'backup': True}})

    @mock.patch('osc_lib.utils.wait_for_status')
    @mock.patch('tunnelclient.osc.v2.utils.get_member_attrs')
    def test_member_set_wait(self, mock_attrs, mock_wait):
        mock_attrs.return_value = {'pool_id': self._mem.pool_id,
                                   'member_id': self._mem.id,
                                   'name': 'new_name'}
        arglist = [self._mem.pool_id, self._mem.id, '--name',
                   'new_name', '--wait']
        verifylist = [
            ('pool', self._mem.pool_id),
            ('member', self._mem.id),
            ('name', 'new_name'),
            ('wait', True),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.api_mock.member_set.assert_called_with(
            pool_id=self._mem.pool_id, member_id=self._mem.id,
            json={'member': {'name': 'new_name'}})
        mock_wait.assert_called_once_with(
            status_f=mock.ANY,
            res_id=self._mem.id,
            sleep_time=mock.ANY,
            status_field='provisioning_status')


class TestMemberShow(TestMember):

    def setUp(self):
        super().setUp()
        self.api_mock.member_show.return_value = self.mem_info
        lb_client = self.app.client_manager
        lb_client.vpn_server = self.api_mock

        self.cmd = member.ShowMember(self.app, None)

    @mock.patch('tunnelclient.osc.v2.utils.get_member_attrs')
    def test_member_show(self, mock_attrs):
        mock_attrs.return_value = {'member_id': self._mem.id,
                                   'pool_id': self._mem.pool_id}
        arglist = [self._mem.pool_id, self._mem.id]
        verifylist = [
            ('pool', self._mem.pool_id),
            ('member', self._mem.id)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.api_mock.member_show.assert_called_with(
            member_id=self._mem.id,
            pool_id=self._mem.pool_id
        )


class TestMemberUnset(TestMember):
    PARAMETERS = ('backup', 'monitor_address', 'monitor_port', 'name',
                  'weight')

    def setUp(self):
        super().setUp()
        self.cmd = member.UnsetMember(self.app, None)

    def test_member_unset_backup(self):
        self._test_member_unset_param('backup')

    def test_member_unset_monitor_address(self):
        self._test_member_unset_param('monitor_address')

    def test_member_unset_monitor_port(self):
        self._test_member_unset_param('monitor_port')

    def test_member_unset_name(self):
        self._test_member_unset_param('name')

    def test_member_unset_name_wait(self):
        self._test_member_unset_param_wait('name')

    def test_member_unset_weight(self):
        self._test_member_unset_param('weight')

    def _test_member_unset_param(self, param):
        self.api_mock.member_set.reset_mock()
        arg_param = param.replace('_', '-') if '_' in param else param
        arglist = [self._mem.pool_id, self._mem.id, '--%s' % arg_param]
        ref_body = {'member': {param: None}}
        verifylist = [
            ('pool', self._mem.pool_id),
            ('member', self._mem.id),
        ]
        for ref_param in self.PARAMETERS:
            verifylist.append((ref_param, param == ref_param))
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.api_mock.member_set.assert_called_once_with(
            pool_id=self._mem.pool_id, member_id=self._mem.id, json=ref_body)

    @mock.patch('osc_lib.utils.wait_for_status')
    def _test_member_unset_param_wait(self, param, mock_wait):
        self.api_mock.member_set.reset_mock()
        arg_param = param.replace('_', '-') if '_' in param else param
        arglist = [self._mem.pool_id, self._mem.id, '--%s' % arg_param,
                   '--wait']
        ref_body = {'member': {param: None}}
        verifylist = [
            ('pool', self._mem.pool_id),
            ('member', self._mem.id),
            ('wait', True),
        ]
        for ref_param in self.PARAMETERS:
            verifylist.append((ref_param, param == ref_param))
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.api_mock.member_set.assert_called_once_with(
            pool_id=self._mem.pool_id, member_id=self._mem.id, json=ref_body)
        mock_wait.assert_called_once_with(
            status_f=mock.ANY,
            res_id=self._mem.id,
            sleep_time=mock.ANY,
            status_field='provisioning_status')

    def test_member_unset_all(self):
        self.api_mock.pool_set.reset_mock()
        ref_body = {'member': {x: None for x in self.PARAMETERS}}
        arglist = [self._mem.pool_id, self._mem.id]
        for ref_param in self.PARAMETERS:
            arg_param = (ref_param.replace('_', '-') if '_' in ref_param else
                         ref_param)
            arglist.append('--%s' % arg_param)
        verifylist = list(zip(self.PARAMETERS, [True] * len(self.PARAMETERS)))
        verifylist = [('member', self._mem.id)] + verifylist
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.api_mock.member_set.assert_called_once_with(
            member_id=self._mem.id, pool_id=self._mem.pool_id, json=ref_body)

    def test_member_unset_none(self):
        self.api_mock.pool_set.reset_mock()
        arglist = [self._mem.pool_id, self._mem.id]
        verifylist = list(zip(self.PARAMETERS, [False] * len(self.PARAMETERS)))
        verifylist = [('member', self._mem.id)] + verifylist
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.api_mock.member_set.assert_not_called()



#   Copyright 2019 Red Hat, Inc. All rights reserved.
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
import copy
from unittest import mock

from osc_lib import exceptions

from tunnelclient.osc.v2 import constants
from tunnelclient.osc.v2 import pool as pool
from tunnelclient.tests.unit.osc.v2 import constants as attr_consts
from tunnelclient.tests.unit.osc.v2 import fakes


class TestPool(fakes.TestTunnelClient):

    def setUp(self):
        super().setUp()

        self._po = fakes.createFakeResource('pool')
        self.pool_info = copy.deepcopy(attr_consts.POOL_ATTRS)
        self.columns = copy.deepcopy(constants.POOL_COLUMNS)

        self.api_mock = mock.Mock()
        self.api_mock.pool_list.return_value = copy.deepcopy(
            {'pools': [attr_consts.POOL_ATTRS]})
        lb_client = self.app.client_manager
        lb_client.vpn_server = self.api_mock


class TestPoolList(TestPool):

    def setUp(self):
        super().setUp()
        self.datalist = (tuple(
            attr_consts.POOL_ATTRS[k] for k in self.columns
        ),)
        self.cmd = pool.ListPool(self.app, None)

    def test_pool_list_no_options(self):
        arglist = []
        verifylist = []

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = self.cmd.take_action(parsed_args)

        self.api_mock.pool_list.assert_called_with()
        self.assertEqual(self.columns, columns)
        self.assertEqual(self.datalist, tuple(data))


class TestPoolDelete(TestPool):

    def setUp(self):
        super().setUp()
        self.cmd = pool.DeletePool(self.app, None)

    def test_pool_delete(self):
        arglist = [self._po.id]
        verifylist = [
            ('pool', self._po.id)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.api_mock.pool_delete.assert_called_with(
            pool_id=self._po.id)

    @mock.patch('osc_lib.utils.wait_for_delete')
    def test_pool_delete_wait(self, mock_wait):
        arglist = [self._po.id, '--wait']
        verifylist = [
            ('pool', self._po.id),
            ('wait', True),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.api_mock.pool_delete.assert_called_with(
            pool_id=self._po.id)
        mock_wait.assert_called_once_with(
            manager=mock.ANY,
            res_id=self._po.id,
            sleep_time=mock.ANY,
            status_field='provisioning_status')

    def test_listener_delete_failure(self):
        arglist = ['unknown_pool']
        verifylist = [
            ('pool', 'unknown_pool')
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.assertRaises(exceptions.CommandError, self.cmd.take_action,
                          parsed_args)
        self.assertNotCalled(self.api_mock.pool_delete)


class TestPoolCreate(TestPool):

    def setUp(self):
        super().setUp()
        self.api_mock.pool_create.return_value = {
            'pool': self.pool_info}
        lb_client = self.app.client_manager
        lb_client.vpn_server = self.api_mock

        self.cmd = pool.CreatePool(self.app, None)

    @mock.patch('tunnelclient.osc.v2.utils.get_pool_attrs')
    def test_pool_create(self, mock_attrs):
        mock_attrs.return_value = self.pool_info
        arglist = ['--vpnserver', 'mock_lb_id',
                   '--name', self._po.name,
                   '--protocol', 'HTTP',
                   '--lb-algorithm', 'ROUND_ROBIN',
                   '--enable-tls',
                   '--tls-container-ref', self._po.tls_container_ref,
                   '--ca-tls-container-ref', self._po.ca_tls_container_ref,
                   '--crl-container-ref', self._po.crl_container_ref,
                   '--tls-ciphers', self._po.tls_ciphers,
                   '--tls-version', self._po.tls_versions[0],
                   '--tls-version', self._po.tls_versions[1]]

        verifylist = [
            ('vpnserver', 'mock_lb_id'),
            ('name', self._po.name),
            ('protocol', 'HTTP'),
            ('lb_algorithm', 'ROUND_ROBIN'),
            ('enable_tls', self._po.tls_enabled),
            ('tls_container_ref', self._po.tls_container_ref),
            ('ca_tls_container_ref', self._po.ca_tls_container_ref),
            ('crl_container_ref', self._po.crl_container_ref),
            ('tls_ciphers', self._po.tls_ciphers),
            ('tls_versions', self._po.tls_versions)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.api_mock.pool_create.assert_called_with(
            json={'pool': self.pool_info})

    @mock.patch('osc_lib.utils.wait_for_status')
    @mock.patch('tunnelclient.osc.v2.utils.get_pool_attrs')
    def test_pool_create_wait(self, mock_attrs, mock_wait):
        self.pool_info['vpnservers'] = [{'id': 'mock_lb_id'}]
        mock_attrs.return_value = self.pool_info
        self.api_mock.pool_show.return_value = self.pool_info
        arglist = ['--vpnserver', 'mock_lb_id',
                   '--name', self._po.name,
                   '--protocol', 'HTTP',
                   '--lb-algorithm', 'ROUND_ROBIN',
                   '--wait']

        verifylist = [
            ('vpnserver', 'mock_lb_id'),
            ('name', self._po.name),
            ('protocol', 'HTTP'),
            ('lb_algorithm', 'ROUND_ROBIN'),
            ('wait', True),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.api_mock.pool_create.assert_called_with(
            json={'pool': self.pool_info})
        mock_wait.assert_called_once_with(
            status_f=mock.ANY,
            res_id='mock_lb_id',
            sleep_time=mock.ANY,
            status_field='provisioning_status')


class TestPoolShow(TestPool):

    def setUp(self):
        super().setUp()
        self.api_mock.pool_show.return_value = self.pool_info
        lb_client = self.app.client_manager
        lb_client.vpn_server = self.api_mock

        self.cmd = pool.ShowPool(self.app, None)

    def test_pool_show(self,):
        arglist = [self._po.id]
        verifylist = [
            ('pool', self._po.id),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.api_mock.pool_show.assert_called_with(pool_id=self._po.id)


class TestPoolSet(TestPool):

    def setUp(self):
        super().setUp()
        self.cmd = pool.SetPool(self.app, None)

    def test_pool_set(self):
        new_tls_id, new_ca_id, new_crl_id = (
            'test-tls-container-id', 'test-ca-tls-container-id',
            'test-crl-container-id')
        arglist = [self._po.id, '--name', 'new_name', '--tls-container-ref',
                   new_tls_id, '--ca-tls-container-ref', new_ca_id,
                   '--crl-container-ref', new_crl_id, '--enable-tls',
                   '--tls-ciphers', self._po.tls_ciphers,
                   '--tls-version', self._po.tls_versions[0],
                   '--tls-version', self._po.tls_versions[1]]
        verifylist = [
            ('pool', self._po.id),
            ('name', 'new_name'),
            ('tls_ciphers', self._po.tls_ciphers),
            ('tls_versions', self._po.tls_versions)
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.api_mock.pool_set.assert_called_with(
            self._po.id, json={'pool': {'name': 'new_name',
                                        'tls_container_ref': new_tls_id,
                                        'ca_tls_container_ref': new_ca_id,
                                        'crl_container_ref': new_crl_id,
                                        'tls_enabled': True,
                                        'tls_ciphers': self._po.tls_ciphers,
                                        'tls_versions': self._po.tls_versions
                                        }})

    @mock.patch('osc_lib.utils.wait_for_status')
    def test_pool_set_wait(self, mock_wait):
        arglist = [self._po.id, '--name', 'new_name', '--wait']
        verifylist = [
            ('pool', self._po.id),
            ('name', 'new_name'),
            ('wait', True),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.api_mock.pool_set.assert_called_with(
            self._po.id, json={'pool': {'name': 'new_name'}})
        mock_wait.assert_called_once_with(
            status_f=mock.ANY,
            res_id=self._po.id,
            sleep_time=mock.ANY,
            status_field='provisioning_status')


class TestPoolUnset(TestPool):
    PARAMETERS = ('name', 'description', 'ca_tls_container_ref',
                  'crl_container_ref', 'session_persistence',
                  'tls_container_ref', 'tls_versions', 'tls_ciphers')

    def setUp(self):
        super().setUp()
        self.cmd = pool.UnsetPool(self.app, None)

    def test_pool_unset_name(self):
        self._test_pool_unset_param('name')

    def test_pool_unset_name_wait(self):
        self._test_pool_unset_param_wait('name')

    def test_pool_unset_description(self):
        self._test_pool_unset_param('description')

    def test_pool_unset_ca_tls_container_ref(self):
        self._test_pool_unset_param('ca_tls_container_ref')

    def test_pool_unset_crl_container_ref(self):
        self._test_pool_unset_param('crl_container_ref')

    def test_pool_unset_session_persistence(self):
        self._test_pool_unset_param('session_persistence')

    def test_pool_unset_tls_container_ref(self):
        self._test_pool_unset_param('tls_container_ref')

    def test_pool_unset_tls_versions(self):
        self._test_pool_unset_param('tls_versions')

    def test_pool_unset_tls_ciphers(self):
        self._test_pool_unset_param('tls_ciphers')

    def _test_pool_unset_param(self, param):
        self.api_mock.pool_set.reset_mock()
        arg_param = param.replace('_', '-') if '_' in param else param
        arglist = [self._po.id, '--%s' % arg_param]
        ref_body = {'pool': {param: None}}
        verifylist = [
            ('pool', self._po.id),
        ]
        for ref_param in self.PARAMETERS:
            verifylist.append((ref_param, param == ref_param))
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.api_mock.pool_set.assert_called_once_with(
            self._po.id, json=ref_body)

    @mock.patch('osc_lib.utils.wait_for_status')
    def _test_pool_unset_param_wait(self, param, mock_wait):
        self.api_mock.pool_set.reset_mock()
        arg_param = param.replace('_', '-') if '_' in param else param
        arglist = [self._po.id, '--%s' % arg_param, '--wait']
        ref_body = {'pool': {param: None}}
        verifylist = [
            ('pool', self._po.id),
            ('wait', True),
        ]
        for ref_param in self.PARAMETERS:
            verifylist.append((ref_param, param == ref_param))
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.api_mock.pool_set.assert_called_once_with(
            self._po.id, json=ref_body)
        mock_wait.assert_called_once_with(
            status_f=mock.ANY,
            res_id=self._po.id,
            sleep_time=mock.ANY,
            status_field='provisioning_status')

    def test_pool_unset_all(self):
        self.api_mock.pool_set.reset_mock()
        ref_body = {'pool': {x: None for x in self.PARAMETERS}}
        arglist = [self._po.id]
        for ref_param in self.PARAMETERS:
            arg_param = (ref_param.replace('_', '-') if '_' in ref_param else
                         ref_param)
            arglist.append('--%s' % arg_param)
        verifylist = list(zip(self.PARAMETERS, [True] * len(self.PARAMETERS)))
        verifylist = [('pool', self._po.id)] + verifylist
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.api_mock.pool_set.assert_called_once_with(
            self._po.id, json=ref_body)

    def test_pool_unset_none(self):
        self.api_mock.pool_set.reset_mock()
        arglist = [self._po.id]
        verifylist = list(zip(self.PARAMETERS, [False] * len(self.PARAMETERS)))
        verifylist = [('pool', self._po.id)] + verifylist
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.api_mock.pool_set.assert_not_called()



# Copyright (c) 2018 China Telecom Corporation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""Tunnel provider action implementation"""

from cliff import lister
from osc_lib import utils

from tunnelclient.osc.v2 import constants as const
from tunnelclient.osc.v2 import utils as v2_utils


class ListProvider(lister.Lister):
    """List all providers"""

    def get_parser(self, prog_name):
        parser = super().get_parser(prog_name)

        return parser

    def take_action(self, parsed_args):
        columns = const.PROVIDER_COLUMNS
        data = self.app.client_manager.vpn_server.provider_list()

        return (columns,
                (utils.get_dict_properties(
                    s, columns,
                    formatters={},
                ) for s in data['providers']))


class ListProviderCapability(lister.Lister):
    """List specified provider driver's capabilities."""

    def get_parser(self, prog_name):
        parser = super().get_parser(prog_name)

        parser.add_argument(
            'provider',
            metavar='<provider_name>',
            help="Name of the provider driver."
        )
        type_group = parser.add_mutually_exclusive_group()
        type_group.add_argument(
            '--flavor',
            action='store_true',
            default=None,
            help="Get capabilities for flavor only."
        )
        type_group.add_argument(
            '--availability-zone',
            action='store_true',
            default=None,
            help="Get capabilities for availability zone only."
        )

        return parser

    def take_action(self, parsed_args):
        columns = const.PROVIDER_CAPABILITY_COLUMNS
        attrs = v2_utils.get_provider_attrs(parsed_args)
        provider = attrs.pop('provider_name')
        fetch_flavor = attrs.pop('flavor', False)
        fetch_az = attrs.pop('availability_zone', False)
        client = self.app.client_manager

        data = []
        if not fetch_az:
            flavor_data = (
                client.vpn_server.
                provider_flavor_capability_list(provider=provider))
            for capability in flavor_data['flavor_capabilities']:
                capability['type'] = 'flavor'
                data.append(capability)
        if not fetch_flavor:
            az_data = (
                client.vpn_server.
                provider_availability_zone_capability_list(
                    provider=provider))
            for capability in az_data['availability_zone_capabilities']:
                capability['type'] = 'availability_zone'
                data.append(capability)
        return (columns,
                (utils.get_dict_properties(
                    s, columns,
                    formatters={},
                ) for s in data))



#
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import copy
from unittest import mock

from osc_lib import exceptions

from tunnelclient.osc.v2 import availabilityzone
from tunnelclient.osc.v2 import constants
from tunnelclient.tests.unit.osc.v2 import constants as attr_consts
from tunnelclient.tests.unit.osc.v2 import fakes


class TestAvailabilityzone(fakes.TestTunnelClient):

    def setUp(self):
        super().setUp()

        self._availabilityzone = fakes.createFakeResource('availability_zone')
        self.availabilityzone_info = copy.deepcopy(
            attr_consts.AVAILABILITY_ZONE_ATTRS)
        self.columns = copy.deepcopy(constants.AVAILABILITYZONE_COLUMNS)

        self.api_mock = mock.Mock()
        self.api_mock.availabilityzone_list.return_value = copy.deepcopy(
            {'availability_zones': [attr_consts.AVAILABILITY_ZONE_ATTRS]})
        lb_client = self.app.client_manager
        lb_client.vpn_server = self.api_mock


class TestAvailabilityzoneList(TestAvailabilityzone):

    def setUp(self):
        super().setUp()
        self.datalist = (tuple(
            attr_consts.AVAILABILITY_ZONE_ATTRS[k] for k in self.columns),)
        self.cmd = availabilityzone.ListAvailabilityzone(self.app, None)

    def test_availabilityzone_list_no_options(self):
        arglist = []
        verifylist = []

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = self.cmd.take_action(parsed_args)

        self.api_mock.availabilityzone_list.assert_called_with()
        self.assertEqual(self.columns, columns)
        self.assertEqual(self.datalist, tuple(data))

    def test_availabilityzone_list_with_options(self):
        arglist = ['--name', 'availabilityzone1']
        verifylist = [('name', 'availabilityzone1')]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        columns, data = self.cmd.take_action(parsed_args)
        self.api_mock.availabilityzone_list.assert_called_with(
            name='availabilityzone1')

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.datalist, tuple(data))


class TestAvailabilityzoneDelete(TestAvailabilityzone):

    def setUp(self):
        super().setUp()
        self.cmd = availabilityzone.DeleteAvailabilityzone(self.app, None)

    def test_availabilityzone_delete(self):
        arglist = [self._availabilityzone.name]
        verifylist = [
            ('availabilityzone', self._availabilityzone.name)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.api_mock.availabilityzone_delete.assert_called_with(
            availabilityzone_name=self._availabilityzone.name)

    def test_availabilityzone_delete_failure(self):
        arglist = ['unknown_availabilityzone']
        verifylist = [
            ('availabilityzone', 'unknown_availabilityzone')
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.assertRaises(exceptions.CommandError, self.cmd.take_action,
                          parsed_args)
        self.assertNotCalled(self.api_mock.availabilityzone_delete)


class TestAvailabilityzoneCreate(TestAvailabilityzone):

    def setUp(self):
        super().setUp()
        self.api_mock.availabilityzone_create.return_value = {
            'availability_zone': self.availabilityzone_info}
        lb_client = self.app.client_manager
        lb_client.vpn_server = self.api_mock

        self.cmd = availabilityzone.CreateAvailabilityzone(self.app, None)

    @mock.patch('tunnelclient.osc.v2.utils.get_availabilityzone_attrs')
    def test_availabilityzone_create(self, mock_client):
        mock_client.return_value = self.availabilityzone_info
        arglist = ['--name', self._availabilityzone.name,
                   '--availabilityzoneprofile', 'mock_azpf_id',
                   '--description', 'description for availabilityzone']
        verifylist = [
            ('availabilityzoneprofile', 'mock_azpf_id'),
            ('name', self._availabilityzone.name),
            ('description', 'description for availabilityzone')
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.api_mock.availabilityzone_create.assert_called_with(
            json={'availability_zone': self.availabilityzone_info})


class TestAvailabilityzoneShow(TestAvailabilityzone):

    def setUp(self):
        super().setUp()
        mock_show = self.api_mock.availabilityzone_show
        mock_show.return_value = self.availabilityzone_info
        lb_client = self.app.client_manager
        lb_client.vpn_server = self.api_mock

        self.cmd = availabilityzone.ShowAvailabilityzone(self.app, None)

    def test_availabilityzone_show(self):
        arglist = [self._availabilityzone.name]
        verifylist = [
            ('availabilityzone', self._availabilityzone.name),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.api_mock.availabilityzone_show.assert_called_with(
            availabilityzone_name=self._availabilityzone.name)


class TestAvailabilityzoneSet(TestAvailabilityzone):

    def setUp(self):
        super().setUp()
        self.cmd = availabilityzone.SetAvailabilityzone(self.app, None)

    def test_availabilityzone_set(self):
        arglist = [self._availabilityzone.name, '--description', 'new_desc']
        verifylist = [
            ('availabilityzone', self._availabilityzone.name),
            ('description', 'new_desc'),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.api_mock.availabilityzone_set.assert_called_with(
            self._availabilityzone.name, json={
                'availability_zone': {
                    'description': 'new_desc'
                }})


class TestAvailabilityzoneUnset(TestAvailabilityzone):
    PARAMETERS = ('description',)

    def setUp(self):
        super().setUp()
        self.cmd = availabilityzone.UnsetAvailabilityzone(self.app, None)

    def test_hm_unset_description(self):
        self._test_availabilityzone_unset_param('description')

    def _test_availabilityzone_unset_param(self, param):
        self.api_mock.availabilityzone_set.reset_mock()
        arg_param = param.replace('_', '-') if '_' in param else param
        arglist = [self._availabilityzone.name, '--%s' % arg_param]
        ref_body = {'availability_zone': {param: None}}
        verifylist = [
            ('availabilityzone', self._availabilityzone.name),
        ]
        for ref_param in self.PARAMETERS:
            verifylist.append((ref_param, param == ref_param))
        print(verifylist)
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.api_mock.availabilityzone_set.assert_called_once_with(
            self._availabilityzone.name, json=ref_body)

    def test_availabilityzone_unset_all(self):
        self.api_mock.availabilityzone_set.reset_mock()
        ref_body = {'availability_zone': {x: None for x in self.PARAMETERS}}
        arglist = [self._availabilityzone.name]
        for ref_param in self.PARAMETERS:
            arg_param = (ref_param.replace('_', '-') if '_' in ref_param else
                         ref_param)
            arglist.append('--%s' % arg_param)
        verifylist = list(zip(self.PARAMETERS, [True] * len(self.PARAMETERS)))
        verifylist = [('availabilityzone',
                       self._availabilityzone.name)] + verifylist
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.api_mock.availabilityzone_set.assert_called_once_with(
            self._availabilityzone.name, json=ref_body)

    def test_availabilityzone_unset_none(self):
        self.api_mock.availabilityzone_set.reset_mock()
        arglist = [self._availabilityzone.name]
        verifylist = list(zip(self.PARAMETERS, [False] * len(self.PARAMETERS)))
        verifylist = [('availabilityzone',
                       self._availabilityzone.name)] + verifylist
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.cmd.take_action(parsed_args)
        self.api_mock.availabilityzone_set.assert_not_called()
